# Stubs for csv (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Iterable

QUOTE_ALL = ...  # type: int
QUOTE_MINIMAL = ...  # type: int
QUOTE_NONE = ...  # type: int
QUOTE_NONNUMERIC = ...  # type: int

class Error(Exception): ...

def writer(csvfile, dialect=..., **fmtparams): ...
def reader(csvfile, dialect=..., **fmtparams): ...
def register_dialect(name, dialect=..., **fmtparams): ...
def unregister_dialect(name): ...
def get_dialect(name): ...
def list_dialects(): ...
def field_size_limit(new_limit=...): ...

class Dialect:
    delimiter = ...  # type: Any
    quotechar = ...  # type: Any
    escapechar = ...  # type: Any
    doublequote = ...  # type: Any
    skipinitialspace = ...  # type: Any
    lineterminator = ...  # type: Any
    quoting = ...  # type: Any
    def __init__(self) -> None: ...

class excel(Dialect):
    delimiter = ...  # type: Any
    quotechar = ...  # type: Any
    doublequote = ...  # type: Any
    skipinitialspace = ...  # type: Any
    lineterminator = ...  # type: Any
    quoting = ...  # type: Any

class excel_tab(excel):
    delimiter = ...  # type: Any

class unix_dialect(Dialect):
    delimiter = ...  # type: Any
    quotechar = ...  # type: Any
    doublequote = ...  # type: Any
    skipinitialspace = ...  # type: Any
    lineterminator = ...  # type: Any
    quoting = ...  # type: Any

class DictReader(Iterable):
    restkey = ...  # type: Any
    restval = ...  # type: Any
    reader = ...  # type: Any
    dialect = ...  # type: Any
    line_num = ...  # type: Any
    fieldnames = ...  # type: Any # Actually a property
    def __init__(self, f, fieldnames=..., restkey=..., restval=..., dialect=...,
                 *args, **kwds): ...
    def __iter__(self): ...
    def __next__(self): ...

class DictWriter:
    fieldnames = ...  # type: Any
    restval = ...  # type: Any
    extrasaction = ...  # type: Any
    writer = ...  # type: Any
    def __init__(self, f, fieldnames, restval=..., extrasaction=..., dialect=..., *args, **kwds) -> None: ...
    def writeheader(self): ...
    def writerow(self, rowdict): ...
    def writerows(self, rowdicts): ...

class Sniffer:
    preferred = ...  # type: Any
    def __init__(self) -> None: ...
    def sniff(self, sample, delimiters=...): ...
    def has_header(self, sample): ...



"""Stub file for the 'cmath' module."""

import sys
from typing import Union, Tuple

e = ...  # type: float
pi = ...  # type: float

_C = Union[float, complex]

def acos(x:_C) -> complex: ...
def acosh(x:_C) -> complex: ...
def asin(x:_C) -> complex: ...
def asinh(x:_C) -> complex: ...
def atan(x:_C) -> complex: ...
def atanh(x:_C) -> complex: ...
def cos(x:_C) -> complex: ...
def cosh(x:_C) -> complex: ...
def exp(x:_C) -> complex: ...
def isinf(z:_C) -> bool: ...
def isnan(z:_C) -> bool: ...
def log(x:_C, base:_C = ...) -> complex: ...
def log10(x:_C) -> complex: ...
def phase(z:_C) -> float: ...
def polar(z:_C) -> Tuple[float, float]: ...
def rect(r:float, phi:float) -> complex: ...
def sin(x:_C) -> complex: ...
def sinh(x:_C) -> complex: ...
def sqrt(x:_C) -> complex: ...
def tan(x:_C) -> complex: ...
def tanh(x:_C) -> complex: ...

if sys.version_info[0] >= 3:
    def isfinite(z:_C) -> bool: ...



from typing import Any, Iterable, TypeVar, Set, Dict, List, TextIO, Union, Tuple, Generic, Callable, Generator

__all__ = ... # type: str

from .events import AbstractEventLoop
from .futures import Future

FIRST_EXCEPTION = 'FIRST_EXCEPTION'
FIRST_COMPLETED = 'FIRST_COMPLETED'
ALL_COMPLETED = 'ALL_COMPLETED'
_T = TypeVar('_T')
def sleep(delay: float, result: _T = ..., loop: AbstractEventLoop = ...) -> Future[_T]: ...
def wait(fs: List[Task[_T]], *, loop: AbstractEventLoop = ...,
    timeout: float = ..., return_when: str = ...) -> Future[Tuple[Set[Future[_T]], Set[Future[_T]]]]: ...
def wait_for(fut: Union[Future[_T], Generator[Any, None, _T]], timeout: float, *, loop: AbstractEventLoop = ...) -> Future[_T]: ...


class Task(Future[_T], Generic[_T]):
    _all_tasks = None  # type: Set[Task]
    _current_tasks = {}  # type: Dict[AbstractEventLoop, Task]
    @classmethod
    def current_task(cls, loop: AbstractEventLoop = ...) -> Task: ...
    @classmethod
    def all_tasks(cls, loop: AbstractEventLoop = ...) -> Set[Task]: ...
    def __init__(self, coro: Union[Future[_T], Generator[Any, None, _T]], *, loop: AbstractEventLoop = ...) -> None: ...
    def __repr__(self) -> str: ...
    def get_stack(self, *, limit: int = ...) -> List[Any]: ...  # return List[stackframe]
    def print_stack(self, *, limit: int = ..., file: TextIO = ...) -> None: ...
    def cancel(self) -> bool: ...
    def _step(self, value: Any = ..., exc: Exception = ...) -> None: ...
    def _wakeup(self, future: Future[Any]) -> None: ...




# Stubs for sqlalchemy.types (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from .sql import type_api
from .sql import sqltypes

TypeEngine = type_api.TypeEngine
TypeDecorator = type_api.TypeDecorator
UserDefinedType = type_api.UserDefinedType
BIGINT = sqltypes.BIGINT
BINARY = sqltypes.BINARY
BLOB = sqltypes.BLOB
BOOLEAN = sqltypes.BOOLEAN
BigInteger = sqltypes.BigInteger
Binary = sqltypes.Binary
Boolean = sqltypes.Boolean
CHAR = sqltypes.CHAR
CLOB = sqltypes.CLOB
Concatenable = sqltypes.Concatenable
DATE = sqltypes.DATE
DATETIME = sqltypes.DATETIME
DECIMAL = sqltypes.DECIMAL
Date = sqltypes.Date
DateTime = sqltypes.DateTime
Enum = sqltypes.Enum
FLOAT = sqltypes.FLOAT
Float = sqltypes.Float
INT = sqltypes.INT
INTEGER = sqltypes.INTEGER
Integer = sqltypes.Integer
Interval = sqltypes.Interval
LargeBinary = sqltypes.LargeBinary
NCHAR = sqltypes.NCHAR
NVARCHAR = sqltypes.NVARCHAR
NUMERIC = sqltypes.NUMERIC
Numeric = sqltypes.Numeric
PickleType = sqltypes.PickleType
REAL = sqltypes.REAL
SMALLINT = sqltypes.SMALLINT
SmallInteger = sqltypes.SmallInteger
String = sqltypes.String
TEXT = sqltypes.TEXT
TIME = sqltypes.TIME
TIMESTAMP = sqltypes.TIMESTAMP
Text = sqltypes.Text
Time = sqltypes.Time
Unicode = sqltypes.Unicode
UnicodeText = sqltypes.UnicodeText
VARBINARY = sqltypes.VARBINARY
VARCHAR = sqltypes.VARCHAR



# Stubs for sqlalchemy.exc (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class SQLAlchemyError(Exception): ...
class ArgumentError(SQLAlchemyError): ...
class NoSuchModuleError(ArgumentError): ...
class NoForeignKeysError(ArgumentError): ...
class AmbiguousForeignKeysError(ArgumentError): ...

class CircularDependencyError(SQLAlchemyError):
    cycles = ... # type: Any
    edges = ... # type: Any
    def __init__(self, message, cycles, edges, msg=...) -> None: ...
    def __reduce__(self): ...

class CompileError(SQLAlchemyError): ...

class UnsupportedCompilationError(CompileError):
    def __init__(self, compiler, element_type) -> None: ...

class IdentifierError(SQLAlchemyError): ...
class DisconnectionError(SQLAlchemyError): ...
class TimeoutError(SQLAlchemyError): ...
class InvalidRequestError(SQLAlchemyError): ...
class NoInspectionAvailable(InvalidRequestError): ...
class ResourceClosedError(InvalidRequestError): ...
class NoSuchColumnError(KeyError, InvalidRequestError): ...
class NoReferenceError(InvalidRequestError): ...

class NoReferencedTableError(NoReferenceError):
    table_name = ... # type: Any
    def __init__(self, message, tname) -> None: ...
    def __reduce__(self): ...

class NoReferencedColumnError(NoReferenceError):
    table_name = ... # type: Any
    column_name = ... # type: Any
    def __init__(self, message, tname, cname) -> None: ...
    def __reduce__(self): ...

class NoSuchTableError(InvalidRequestError): ...
class UnboundExecutionError(InvalidRequestError): ...
class DontWrapMixin: ...

UnmappedColumnError = ... # type: Any

class StatementError(SQLAlchemyError):
    statement = ... # type: Any
    params = ... # type: Any
    orig = ... # type: Any
    detail = ... # type: Any
    def __init__(self, message, statement, params, orig) -> None: ...
    def add_detail(self, msg): ...
    def __reduce__(self): ...
    def __unicode__(self): ...

class DBAPIError(StatementError):
    @classmethod
    def instance(cls, statement, params, orig, dbapi_base_err, connection_invalidated=..., dialect=...): ...
    def __reduce__(self): ...
    connection_invalidated = ... # type: Any
    def __init__(self, statement, params, orig, connection_invalidated=...) -> None: ...

class InterfaceError(DBAPIError): ...
class DatabaseError(DBAPIError): ...
class DataError(DatabaseError): ...
class OperationalError(DatabaseError): ...
class IntegrityError(DatabaseError): ...
class InternalError(DatabaseError): ...
class ProgrammingError(DatabaseError): ...
class NotSupportedError(DatabaseError): ...
class SADeprecationWarning(DeprecationWarning): ...
class SAPendingDeprecationWarning(PendingDeprecationWarning): ...
class SAWarning(RuntimeWarning): ...



# Stubs for kazoo.exceptions (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class KazooException(Exception): ...
class ZookeeperError(KazooException): ...
class CancelledError(KazooException): ...
class ConfigurationError(KazooException): ...
class ZookeeperStoppedError(KazooException): ...
class ConnectionDropped(KazooException): ...
class LockTimeout(KazooException): ...
class WriterNotClosedException(KazooException): ...

EXCEPTIONS = ... # type: Any

class RolledBackError(ZookeeperError): ...
class SystemZookeeperError(ZookeeperError): ...
class RuntimeInconsistency(ZookeeperError): ...
class DataInconsistency(ZookeeperError): ...
class ConnectionLoss(ZookeeperError): ...
class MarshallingError(ZookeeperError): ...
class UnimplementedError(ZookeeperError): ...
class OperationTimeoutError(ZookeeperError): ...
class BadArgumentsError(ZookeeperError): ...
class NewConfigNoQuorumError(ZookeeperError): ...
class ReconfigInProcessError(ZookeeperError): ...
class APIError(ZookeeperError): ...
class NoNodeError(ZookeeperError): ...
class NoAuthError(ZookeeperError): ...
class BadVersionError(ZookeeperError): ...
class NoChildrenForEphemeralsError(ZookeeperError): ...
class NodeExistsError(ZookeeperError): ...
class NotEmptyError(ZookeeperError): ...
class SessionExpiredError(ZookeeperError): ...
class InvalidCallbackError(ZookeeperError): ...
class InvalidACLError(ZookeeperError): ...
class AuthFailedError(ZookeeperError): ...
class SessionMovedError(ZookeeperError): ...
class NotReadOnlyCallError(ZookeeperError): ...
class ConnectionClosedError(SessionExpiredError): ...

ConnectionLossException = ... # type: Any
MarshallingErrorException = ... # type: Any
SystemErrorException = ... # type: Any
RuntimeInconsistencyException = ... # type: Any
DataInconsistencyException = ... # type: Any
UnimplementedException = ... # type: Any
OperationTimeoutException = ... # type: Any
BadArgumentsException = ... # type: Any
ApiErrorException = ... # type: Any
NoNodeException = ... # type: Any
NoAuthException = ... # type: Any
BadVersionException = ... # type: Any
NoChildrenForEphemeralsException = ... # type: Any
NodeExistsException = ... # type: Any
InvalidACLException = ... # type: Any
AuthFailedException = ... # type: Any
NotEmptyException = ... # type: Any
SessionExpiredException = ... # type: Any
InvalidCallbackException = ... # type: Any



"""Fix up various things after deserialization."""

from typing import Any, Dict, Optional, cast

from mypy.nodes import (MypyFile, SymbolNode, SymbolTable, SymbolTableNode,
                        TypeInfo, FuncDef, OverloadedFuncDef, Decorator, Var,
                        TypeVarExpr, ClassDef,
                        LDEF, MDEF, GDEF, MODULE_REF)
from mypy.types import (CallableType, EllipsisType, Instance, Overloaded, TupleType,
                        TypeList, TypeVarType, UnboundType, UnionType, TypeVisitor)
from mypy.visitor import NodeVisitor


def fixup_module_pass_one(tree: MypyFile, modules: Dict[str, MypyFile]) -> None:
    node_fixer = NodeFixer(modules)
    node_fixer.visit_symbol_table(tree.names)


def fixup_module_pass_two(tree: MypyFile, modules: Dict[str, MypyFile]) -> None:
    compute_all_mros(tree.names, modules)


def compute_all_mros(symtab: SymbolTable, modules: Dict[str, MypyFile]) -> None:
    for key, value in symtab.items():
        if value.kind in (LDEF, MDEF, GDEF) and isinstance(value.node, TypeInfo):
            info = value.node
            info.calculate_mro()
            assert info.mro, "No MRO calculated for %s" % (info.fullname(),)
            compute_all_mros(info.names, modules)


# TODO: Fix up .info when deserializing, i.e. much earlier.
class NodeFixer(NodeVisitor[None]):
    current_info = None  # type: Optional[TypeInfo]

    def __init__(self, modules: Dict[str, MypyFile], type_fixer: 'TypeFixer' = None) -> None:
        self.modules = modules
        if type_fixer is None:
            type_fixer = TypeFixer(self.modules)
        self.type_fixer = type_fixer

    # NOTE: This method isn't (yet) part of the NodeVisitor API.
    def visit_type_info(self, info: TypeInfo) -> None:
        save_info = self.current_info
        try:
            self.current_info = info
            if info.defn:
                info.defn.accept(self)
            if info.names:
                self.visit_symbol_table(info.names)
            if info.subtypes:
                for st in info.subtypes:
                    self.visit_type_info(st)
            if info.bases:
                for base in info.bases:
                    base.accept(self.type_fixer)
            if info._promote:
                info._promote.accept(self.type_fixer)
            if info.tuple_type:
                info.tuple_type.accept(self.type_fixer)
        finally:
            self.current_info = save_info

    # NOTE: This method *definitely* isn't part of the NodeVisitor API.
    def visit_symbol_table(self, symtab: SymbolTable) -> None:
        # Copy the items because we may mutate symtab.
        for key, value in list(symtab.items()):
            cross_ref = value.cross_ref
            if cross_ref is not None:  # Fix up cross-reference.
                del value.cross_ref
                if cross_ref in self.modules:
                    value.node = self.modules[cross_ref]
                else:
                    stnode = lookup_qualified_stnode(self.modules, cross_ref)
                    assert stnode is not None, "Could not find cross-ref %s" % (cross_ref,)
                    value.node = stnode.node
                    value.type_override = stnode.type_override
            else:
                if isinstance(value.node, TypeInfo):
                    # TypeInfo has no accept().  TODO: Add it?
                    self.visit_type_info(value.node)
                elif value.node is not None:
                    value.node.accept(self)
                if value.type_override is not None:
                    value.type_override.accept(self.type_fixer)

    def visit_func_def(self, func: FuncDef) -> None:
        if self.current_info is not None:
            func.info = self.current_info
        if func.type is not None:
            func.type.accept(self.type_fixer)
        for arg in func.arguments:
            if arg.type_annotation is not None:
                arg.type_annotation.accept(self.type_fixer)

    def visit_overloaded_func_def(self, o: OverloadedFuncDef) -> None:
        if self.current_info is not None:
            o.info = self.current_info
        if o.type:
            o.type.accept(self.type_fixer)
        for item in o.items:
            item.accept(self)

    def visit_decorator(self, d: Decorator) -> None:
        if self.current_info is not None:
            d.var.info = self.current_info
        if d.func:
            d.func.accept(self)
        if d.var:
            d.var.accept(self)
        for node in d.decorators:
            node.accept(self)

    def visit_class_def(self, c: ClassDef) -> None:
        for v in c.type_vars:
            for value in v.values:
                value.accept(self.type_fixer)
            v.upper_bound.accept(self.type_fixer)

    def visit_type_var_expr(self, tv: TypeVarExpr) -> None:
        for value in tv.values:
            value.accept(self.type_fixer)

    def visit_var(self, v: Var) -> None:
        if self.current_info is not None:
            v.info = self.current_info
        if v.type is not None:
            v.type.accept(self.type_fixer)


class TypeFixer(TypeVisitor[None]):
    def __init__(self, modules: Dict[str, MypyFile]) -> None:
        self.modules = modules

    def visit_instance(self, inst: Instance) -> None:
        # TODO: Combine Instances that are exactly the same?
        type_ref = inst.type_ref
        if type_ref is None:
            return  # We've already been here.
        del inst.type_ref
        node = lookup_qualified(self.modules, type_ref)
        if isinstance(node, TypeInfo):
            inst.type = node
            # TODO: Is this needed or redundant?
            # Also fix up the bases, just in case.
            for base in inst.type.bases:
                if base.type is None:
                    base.accept(self)
        for a in inst.args:
            a.accept(self)

    def visit_any(self, o: Any) -> None:
        pass  # Nothing to descend into.

    def visit_callable_type(self, ct: CallableType) -> None:
        if ct.fallback:
            ct.fallback.accept(self)
        for argt in ct.arg_types:
            # argt may be None, e.g. for __self in NamedTuple constructors.
            if argt is not None:
                argt.accept(self)
        if ct.ret_type is not None:
            ct.ret_type.accept(self)
        for v in ct.variables:
            if v.values:
                for val in v.values:
                    val.accept(self)
            v.upper_bound.accept(self)

    def visit_ellipsis_type(self, e: EllipsisType) -> None:
        pass  # Nothing to descend into.

    def visit_overloaded(self, t: Overloaded) -> None:
        for ct in t.items():
            ct.accept(self)

    def visit_deleted_type(self, o: Any) -> None:
        pass  # Nothing to descend into.

    def visit_none_type(self, o: Any) -> None:
        pass  # Nothing to descend into.

    def visit_partial_type(self, o: Any) -> None:
        raise RuntimeError("Shouldn't get here", o)

    def visit_tuple_type(self, tt: TupleType) -> None:
        if tt.items:
            for it in tt.items:
                it.accept(self)
        if tt.fallback is not None:
            tt.fallback.accept(self)

    def visit_type_list(self, tl: TypeList) -> None:
        for t in tl.items:
            t.accept(self)

    def visit_type_var(self, tvt: TypeVarType) -> None:
        if tvt.values:
            for vt in tvt.values:
                vt.accept(self)
        if tvt.upper_bound is not None:
            tvt.upper_bound.accept(self)

    def visit_unbound_type(self, o: UnboundType) -> None:
        for a in o.args:
            a.accept(self)

    def visit_union_type(self, ut: UnionType) -> None:
        if ut.items:
            for it in ut.items:
                it.accept(self)

    def visit_void(self, o: Any) -> None:
        pass  # Nothing to descend into.


def lookup_qualified(modules: Dict[str, MypyFile], name: str) -> SymbolNode:
    stnode = lookup_qualified_stnode(modules, name)
    if stnode is None:
        return None
    else:
        return stnode.node


def lookup_qualified_stnode(modules: Dict[str, MypyFile], name: str) -> SymbolTableNode:
    head = name
    rest = []
    while True:
        assert '.' in head, "Cannot find %s" % (name,)
        head, tail = head.rsplit('.', 1)
        mod = modules.get(head)
        if mod is not None:
            rest.append(tail)
            break
    names = mod.names
    while True:
        assert rest, "Cannot find %s" % (name,)
        key = rest.pop()
        assert key in names, "Cannot find %s for %s" % (key, name)
        stnode = names[key]
        if not rest:
            return stnode
        node = stnode.node
        assert isinstance(node, TypeInfo)
        names = cast(TypeInfo, node).names


def store_qualified(modules: Dict[str, MypyFile], name: str, info: SymbolNode) -> None:
    head = name
    rest = []
    while True:
        head, tail = head.rsplit('.', 1)
        mod = modules.get(head)
        if mod is not None:
            rest.append(tail)
            break
    names = mod.names
    while True:
        assert rest, "Cannot find %s" % (name,)
        key = rest.pop()
        if key not in names:
            assert not rest, "Cannot find %s for %s" % (key, name)
            # Store it.
            # TODO: kind might be something else?
            names[key] = SymbolTableNode(GDEF, info)
            return
        stnode = names[key]
        node = stnode.node
        if not rest:
            stnode.node = info
            return
        assert isinstance(node, TypeInfo)
        names = cast(TypeInfo, node).names



# Stubs for smtplib (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class SMTPException(OSError): ...
class SMTPServerDisconnected(SMTPException): ...

class SMTPResponseException(SMTPException):
    smtp_code = ... # type: Any
    smtp_error = ... # type: Any
    args = ... # type: Any
    def __init__(self, code, msg) -> None: ...

class SMTPSenderRefused(SMTPResponseException):
    smtp_code = ... # type: Any
    smtp_error = ... # type: Any
    sender = ... # type: Any
    args = ... # type: Any
    def __init__(self, code, msg, sender) -> None: ...

class SMTPRecipientsRefused(SMTPException):
    recipients = ... # type: Any
    args = ... # type: Any
    def __init__(self, recipients) -> None: ...

class SMTPDataError(SMTPResponseException): ...
class SMTPConnectError(SMTPResponseException): ...
class SMTPHeloError(SMTPResponseException): ...
class SMTPAuthenticationError(SMTPResponseException): ...

def quoteaddr(addrstring): ...
def quotedata(data): ...

class SMTP:
    debuglevel = ... # type: Any
    file = ... # type: Any
    helo_resp = ... # type: Any
    ehlo_msg = ... # type: Any
    ehlo_resp = ... # type: Any
    does_esmtp = ... # type: Any
    default_port = ... # type: Any
    timeout = ... # type: Any
    esmtp_features = ... # type: Any
    source_address = ... # type: Any
    local_hostname = ... # type: Any
    def __init__(self, host=..., port=..., local_hostname=..., timeout=...,
                 source_address=...): ...
    def __enter__(self): ...
    def __exit__(self, *args): ...
    def set_debuglevel(self, debuglevel): ...
    sock = ... # type: Any
    def connect(self, host=..., port=..., source_address=...): ...
    def send(self, s): ...
    def putcmd(self, cmd, args=...): ...
    def getreply(self): ...
    def docmd(self, cmd, args=...): ...
    def helo(self, name=...): ...
    def ehlo(self, name=...): ...
    def has_extn(self, opt): ...
    def help(self, args=...): ...
    def rset(self): ...
    def noop(self): ...
    def mail(self, sender, options=...): ...
    def rcpt(self, recip, options=...): ...
    def data(self, msg): ...
    def verify(self, address): ...
    vrfy = ... # type: Any
    def expn(self, address): ...
    def ehlo_or_helo_if_needed(self): ...
    def login(self, user, password): ...
    def starttls(self, keyfile=..., certfile=..., context=...): ...
    def sendmail(self, from_addr, to_addrs, msg, mail_options=...,
                 rcpt_options=...): ...
    def send_message(self, msg, from_addr=..., to_addrs=..., mail_options=...,
                     rcpt_options=...): ...
    def close(self): ...
    def quit(self): ...

class SMTP_SSL(SMTP):
    default_port = ... # type: Any
    keyfile = ... # type: Any
    certfile = ... # type: Any
    context = ... # type: Any
    def __init__(self, host=..., port=..., local_hostname=..., keyfile=..., certfile=...,
                 timeout=..., source_address=..., context=...): ...

class LMTP(SMTP):
    ehlo_msg = ... # type: Any
    def __init__(self, host=..., port=..., local_hostname=..., source_address=...) -> None: ...
    sock = ... # type: Any
    file = ... # type: Any
    def connect(self, host=..., port=..., source_address=...): ...



# Stubs for uuid

from typing import Tuple

Int = __builtins__.int
Bytes = __builtins__.bytes
FieldsType = Tuple[Int, Int, Int, Int, Int, Int]

class UUID:
    def __init__(self, hex: str=..., bytes: Bytes=..., bytes_le: Bytes=..., fields: FieldsType=..., int: Int=..., version: Int=...) -> None: ...

    @property
    def bytes(self) -> Bytes: ...

    @property
    def bytes_le(self) -> Bytes: ...

    @property
    def clock_seq(self) -> Int: ...

    @property
    def clock_seq_hi_variant(self) -> Int: ...

    @property
    def clock_seq_low(self) -> Int: ...

    @property
    def fields(self) -> FieldsType: ...

    @property
    def hex(self) -> str: ...

    @property
    def int(self) -> Int: ...

    @property
    def node(self) -> Int: ...

    @property
    def time(self) -> Int: ...

    @property
    def time_hi_version(self) -> Int: ...

    @property
    def time_low(self) -> Int: ...

    @property
    def time_mid(self) -> Int: ...

    @property
    def urn(self) -> str: ...

    @property
    def variant(self) -> str: ...

    @property
    def version(self) -> str: ...

def getnode() -> Int: ...
def uuid1(node: Int=..., clock_seq: Int=...) -> UUID: ...
def uuid3(namespace: UUID, name: str) -> UUID: ...
def uuid4() -> UUID: ...
def uuid5(namespace: UUID, name: str) -> UUID: ...

NAMESPACE_DNS = ... # type: UUID
NAMESPACE_URL = ... # type: UUID
NAMESPACE_OID = ... # type: UUID
NAMESPACE_X500 = ... # type: UUID
RESERVED_NCS = ... # type: str
RFC_4122 = ... # type: str
RESERVED_MICROSOFT = ... # type: str
RESERVED_FUTURE = ... # type: str



from typing import Any, Tuple

__all__ = None # type: list[str]
DEFAULTSECT = None # type: str
MAX_INTERPOLATION_DEPTH = None # type: int

class Error(Exception):
    message = None # type: Any
    def __init__(self, msg: str = ...) -> None: ...
    def _get_message(self) -> None: ...
    def _set_message(self, value: str) -> None: ...
    def __repr__(self) -> str: ...
    __str__ = __repr__

class NoSectionError(Error):
    section = ... # type: str
    args = ... # type: Tuple[str]
    def __init__(self, section: str) -> None: ...

class DuplicateSectionError(Error):
    section = ... # type: str
    args = ... # type: Tuple[str]
    def __init__(self, section: str) -> None: ...

class NoOptionError(Error):
    section = ... # type: str
    option = ... # type: str
    args = ... # type: Tuple[str,str]
    def __init__(self, option: str, section: str) -> None: ...

class InterpolationError(Error):
    section = ... # type: str
    option = ... # type: str
    msg = ... # type: str
    args = ... # type: Tuple[str,str,str]
    def __init__(self, option: str, section: str, msg: str) -> None: ...

class InterpolationMissingOptionError(InterpolationError):
    reference = ... # type: str
    args = ... # type: Tuple[str,str,str,str]
    def __init__(self, option: str, section: str, rawval: str, reference: str) -> None: ...

class InterpolationSyntaxError(InterpolationError): ...

class InterpolationDepthError(InterpolationError):
    def __init__(self, option: str, section: str, rawval: str) -> None: ...

class ParsingError(Error):
    filename = ... # type: str
    errors = ... # type: list[Tuple[Any,Any]]
    args = ... # type: Tuple[str]
    def __init__(self, filename: str) -> None: ...
    def append(self, lineno: Any, line: Any) -> None: ...

class MissingSectionHeaderError(ParsingError):
    lineno = ... # type: Any
    line = ... # type: Any
    args = ... # type: Tuple[str,Any,Any]
    def __init__(self, filename: str, lineno: Any, line: Any) -> None: ...


class RawConfigParser:
    _dict = ... # type: Any
    _sections = ... # type: dict
    _defaults = ... # type: dict
    _optcre = ... # type: Any
    SECTCRE = ... # type: Any
    OPTCRE = ... # type: Any
    OPTCRE_NV = ... # type: Any
    def __init__(self, defaults: dict[Any,Any] = ..., dict_type: Any = ..., allow_no_value: bool = ...) -> None: ...
    def defaults(self) -> dict[Any,Any]: ...
    def sections(self) -> list[str]: ...
    def add_section(self, section: str) -> None: ...
    def has_section(self, section: str) -> bool: ...
    def options(self, section: str) -> list[str]: ...
    def read(self, filenames: str) -> list[str]: ...
    def readfp(self, fp: file, filename: str = ...) -> None: ...
    def get(self, section: str, option: str) -> str: ...
    def items(self, section: str) -> list[Tuple[Any,Any]]: ...
    def _get(self, section: str, conv: type, option: str) -> Any: ...
    def getint(self, section: str, option: str) -> int: ...
    def getfloat(self, section: str, option: str) -> float: ...
    _boolean_states = ... # type: dict[str,bool]
    def getboolean(self, section: str, option: str) -> bool: ...
    def optionxform(self, optionstr: str) -> str: ...
    def has_option(self, section: str, option: str) -> bool: ...
    def set(self, section: str, option: str, value: Any = ...) -> None: ...
    def write(self, fp: file) -> None: ...
    def remove_option(self, section: str, option: Any) -> bool: ...
    def remove_section(self, section: str) -> bool: ...
    def _read(self, fp: file, fpname: str) -> None: ...

class ConfigParser(RawConfigParser):
    _KEYCRE = ... # type: Any
    def get(self, section: str, option: str, raw: bool = ..., vars: dict = ...) -> Any: ...
    def items(self, section: str, raw: bool = ..., vars: dict = ...) -> list[Tuple[str,Any]]: ...
    def _interpolate(self, section: str, option: str, rawval: Any, vars: Any) -> str: ...
    def _interpolation_replace(self, match: Any) -> str: ...

class SafeConfigParser(ConfigParser):
    _interpvar_re = ... # type: Any
    def _interpolate(self, section: str, option: str, rawval: Any, vars: Any) -> str: ...
    def _interpolate_some(self, option: str, accum: list, rest: str, section: str, map: dict, depth: int) -> None: ...



# Stubs for email.generator (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class Generator:
    maxheaderlen = ...  # type: Any
    policy = ...  # type: Any
    def __init__(self, outfp, mangle_from_=..., maxheaderlen=..., *, policy=...) -> None: ...
    def write(self, s): ...
    def flatten(self, msg, unixfrom=..., linesep=...): ...
    def clone(self, fp): ...

class BytesGenerator(Generator):
    def write(self, s): ...

class DecodedGenerator(Generator):
    def __init__(self, outfp, mangle_from_=..., maxheaderlen=..., fmt=...) -> None: ...



# Stubs for logging.handlers (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
import logging

threading = ...  # type: Any
DEFAULT_TCP_LOGGING_PORT = ...  # type: Any
DEFAULT_UDP_LOGGING_PORT = ...  # type: Any
DEFAULT_HTTP_LOGGING_PORT = ...  # type: Any
DEFAULT_SOAP_LOGGING_PORT = ...  # type: Any
SYSLOG_UDP_PORT = ...  # type: Any
SYSLOG_TCP_PORT = ...  # type: Any

class BaseRotatingHandler(logging.FileHandler):
    mode = ...  # type: Any
    encoding = ...  # type: Any
    namer = ...  # type: Any
    rotator = ...  # type: Any
    def __init__(self, filename, mode, encoding=..., delay=...) -> None: ...
    def emit(self, record): ...
    def rotation_filename(self, default_name): ...
    def rotate(self, source, dest): ...

class RotatingFileHandler(BaseRotatingHandler):
    maxBytes = ...  # type: Any
    backupCount = ...  # type: Any
    def __init__(self, filename, mode=..., maxBytes=..., backupCount=..., encoding=...,
                 delay=...): ...
    stream = ...  # type: Any
    def doRollover(self): ...
    def shouldRollover(self, record): ...

class TimedRotatingFileHandler(BaseRotatingHandler):
    when = ...  # type: Any
    backupCount = ...  # type: Any
    utc = ...  # type: Any
    atTime = ...  # type: Any
    interval = ...  # type: Any
    suffix = ...  # type: Any
    extMatch = ...  # type: Any
    dayOfWeek = ...  # type: Any
    rolloverAt = ...  # type: Any
    def __init__(self, filename, when=..., interval=..., backupCount=..., encoding=..., delay=...,
                 utc=..., atTime=...): ...
    def computeRollover(self, currentTime): ...
    def shouldRollover(self, record): ...
    def getFilesToDelete(self): ...
    stream = ...  # type: Any
    def doRollover(self): ...

class WatchedFileHandler(logging.FileHandler):
    def __init__(self, filename, mode=..., encoding=..., delay=...) -> None: ...
    stream = ...  # type: Any
    def emit(self, record): ...

class SocketHandler(logging.Handler):
    host = ...  # type: Any
    port = ...  # type: Any
    address = ...  # type: Any
    sock = ...  # type: Any
    closeOnError = ...  # type: Any
    retryTime = ...  # type: Any
    retryStart = ...  # type: Any
    retryMax = ...  # type: Any
    retryFactor = ...  # type: Any
    def __init__(self, host, port) -> None: ...
    def makeSocket(self, timeout=...): ...
    retryPeriod = ...  # type: Any
    def createSocket(self): ...
    def send(self, s): ...
    def makePickle(self, record): ...
    def handleError(self, record): ...
    def emit(self, record): ...
    def close(self): ...

class DatagramHandler(SocketHandler):
    closeOnError = ...  # type: Any
    def __init__(self, host, port) -> None: ...
    def makeSocket(self, timeout=...): ... # TODO: Actually does not have the timeout argument.
    def send(self, s): ...

class SysLogHandler(logging.Handler):
    LOG_EMERG = ...  # type: Any
    LOG_ALERT = ...  # type: Any
    LOG_CRIT = ...  # type: Any
    LOG_ERR = ...  # type: Any
    LOG_WARNING = ...  # type: Any
    LOG_NOTICE = ...  # type: Any
    LOG_INFO = ...  # type: Any
    LOG_DEBUG = ...  # type: Any
    LOG_KERN = ...  # type: Any
    LOG_USER = ...  # type: Any
    LOG_MAIL = ...  # type: Any
    LOG_DAEMON = ...  # type: Any
    LOG_AUTH = ...  # type: Any
    LOG_SYSLOG = ...  # type: Any
    LOG_LPR = ...  # type: Any
    LOG_NEWS = ...  # type: Any
    LOG_UUCP = ...  # type: Any
    LOG_CRON = ...  # type: Any
    LOG_AUTHPRIV = ...  # type: Any
    LOG_FTP = ...  # type: Any
    LOG_LOCAL0 = ...  # type: Any
    LOG_LOCAL1 = ...  # type: Any
    LOG_LOCAL2 = ...  # type: Any
    LOG_LOCAL3 = ...  # type: Any
    LOG_LOCAL4 = ...  # type: Any
    LOG_LOCAL5 = ...  # type: Any
    LOG_LOCAL6 = ...  # type: Any
    LOG_LOCAL7 = ...  # type: Any
    priority_names = ...  # type: Any
    facility_names = ...  # type: Any
    priority_map = ...  # type: Any
    address = ...  # type: Any
    facility = ...  # type: Any
    socktype = ...  # type: Any
    unixsocket = ...  # type: Any
    socket = ...  # type: Any
    formatter = ...  # type: Any
    def __init__(self, address=..., facility=..., socktype=...) -> None: ...
    def encodePriority(self, facility, priority): ...
    def close(self): ...
    def mapPriority(self, levelName): ...
    ident = ...  # type: Any
    append_nul = ...  # type: Any
    def emit(self, record): ...

class SMTPHandler(logging.Handler):
    username = ...  # type: Any
    fromaddr = ...  # type: Any
    toaddrs = ...  # type: Any
    subject = ...  # type: Any
    secure = ...  # type: Any
    timeout = ...  # type: Any
    def __init__(self, mailhost, fromaddr, toaddrs, subject, credentials=..., secure=...,
                 timeout=...): ...
    def getSubject(self, record): ...
    def emit(self, record): ...

class NTEventLogHandler(logging.Handler):
    appname = ...  # type: Any
    dllname = ...  # type: Any
    logtype = ...  # type: Any
    deftype = ...  # type: Any
    typemap = ...  # type: Any
    def __init__(self, appname, dllname=..., logtype=...) -> None: ...
    def getMessageID(self, record): ...
    def getEventCategory(self, record): ...
    def getEventType(self, record): ...
    def emit(self, record): ...
    def close(self): ...

class HTTPHandler(logging.Handler):
    host = ...  # type: Any
    url = ...  # type: Any
    method = ...  # type: Any
    secure = ...  # type: Any
    credentials = ...  # type: Any
    def __init__(self, host, url, method=..., secure=..., credentials=...) -> None: ...
    def mapLogRecord(self, record): ...
    def emit(self, record): ...

class BufferingHandler(logging.Handler):
    capacity = ...  # type: Any
    buffer = ...  # type: Any
    def __init__(self, capacity) -> None: ...
    def shouldFlush(self, record): ...
    def emit(self, record): ...
    def flush(self): ...
    def close(self): ...

class MemoryHandler(BufferingHandler):
    flushLevel = ...  # type: Any
    target = ...  # type: Any
    def __init__(self, capacity, flushLevel=..., target=...) -> None: ...
    def shouldFlush(self, record): ...
    def setTarget(self, target): ...
    buffer = ...  # type: Any
    def flush(self): ...
    def close(self): ...

class QueueHandler(logging.Handler):
    queue = ...  # type: Any
    def __init__(self, queue) -> None: ...
    def enqueue(self, record): ...
    def prepare(self, record): ...
    def emit(self, record): ...

class QueueListener:
    queue = ...  # type: Any
    handlers = ...  # type: Any
    def __init__(self, queue, *handlers) -> None: ...
    def dequeue(self, block): ...
    def start(self): ...
    def prepare(self, record): ...
    def handle(self, record): ...
    def enqueue_sentinel(self): ...
    def stop(self): ...



# Stubs for yaml.dumper (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from yaml.emitter import Emitter
from yaml.serializer import Serializer
from yaml.representer import BaseRepresenter, Representer, SafeRepresenter
from yaml.resolver import BaseResolver, Resolver

class BaseDumper(Emitter, Serializer, BaseRepresenter, BaseResolver):
    def __init__(self, stream, default_style=..., default_flow_style=..., canonical=..., indent=..., width=..., allow_unicode=..., line_break=..., encoding=..., explicit_start=..., explicit_end=..., version=..., tags=...) -> None: ...

class SafeDumper(Emitter, Serializer, SafeRepresenter, Resolver):
    def __init__(self, stream, default_style=..., default_flow_style=..., canonical=..., indent=..., width=..., allow_unicode=..., line_break=..., encoding=..., explicit_start=..., explicit_end=..., version=..., tags=...) -> None: ...

class Dumper(Emitter, Serializer, Representer, Resolver):
    def __init__(self, stream, default_style=..., default_flow_style=..., canonical=..., indent=..., width=..., allow_unicode=..., line_break=..., encoding=..., explicit_start=..., explicit_end=..., version=..., tags=...) -> None: ...



"""Stubs for the binascii module."""

def a2b_base64(string: str) -> str: ...
def a2b_hex(hexstr: str) -> str: ...
def a2b_hqx(string: str) -> str: ...
def a2b_qp(string: str, header: bool = ...) -> str: ...
def a2b_uu(string: str) -> str: ...
def b2a_base64(data: str) -> str: ...
def b2a_hex(data: str) -> str: ...
def b2a_hqx(data: str) -> str: ...
def b2a_qp(data: str, quotetabs: bool = ..., istext: bool = ..., header: bool = ...) -> str: ...
def b2a_uu(data: str) -> str: ...
def crc32(data: str, crc: int = None) -> int: ...
def crc_hqx(data: str, oldcrc: int) -> int: ...
def hexlify(data: str) -> str: ...
def rlecode_hqx(data: str) -> str: ...
def rledecode_hqx(data: str) -> str: ...
def unhexlify(hexstr: str) -> str: ...

class Error(Exception): ...
class Incomplete(Exception): ...



# Generated by stubtool 0.1, DO NOT EDIT
# See https://github.com/o11c/stubtool
#
# Stubs for six.moves.urllib_response (Python 2.7)

from six.moves.urllib.response import (
    addbase as addbase,
    addclosehook as addclosehook,
    addinfo as addinfo,
    addinfourl as addinfourl,
)



# Stubs for posixpath (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from genericpath import *

curdir = ... # type: Any
pardir = ... # type: Any
extsep = ... # type: Any
sep = ... # type: Any
pathsep = ... # type: Any
defpath = ... # type: Any
altsep = ... # type: Any
devnull = ... # type: Any

def normcase(s): ...
def isabs(s): ...
def join(a, *p): ...
def split(p): ...
def splitext(p): ...
def splitdrive(p): ...
def basename(p): ...
def dirname(p): ...
def islink(path): ...
def lexists(path): ...
def samefile(f1, f2): ...
def sameopenfile(fp1, fp2): ...
def samestat(s1, s2): ...
def ismount(path): ...
def walk(top, func, arg): ...
def expanduser(path): ...
def expandvars(path): ...
def normpath(path): ...
def abspath(path): ...
def realpath(filename): ...

supports_unicode_filenames = ... # type: Any

def relpath(path, start=...): ...

# Names in __all__ with no definition:
#   commonprefix
#   exists
#   getatime
#   getctime
#   getmtime
#   getsize
#   isdir
#   isfile



# Python 3.5 ast

import typing
from typing import Any, Union, Iterator

from _ast import (
    Add, alias, And, arg, arguments, Assert, Assign, AST, AsyncFor,
    AsyncFunctionDef, AsyncWith, Attribute, AugAssign, AugLoad, AugStore,
    Await, BinOp, BitAnd, BitOr, BitXor, BoolOp, boolop, Break, Bytes, Call,
    ClassDef, cmpop, Compare, comprehension, Continue, Del, Delete, Dict,
    DictComp, Div, Ellipsis, Eq, ExceptHandler, Expr, expr, Expression,
    expr_context, ExtSlice, FloorDiv, For, FunctionDef, GeneratorExp, Global,
    Gt, GtE, If, IfExp, Import, ImportFrom, In, Index, Interactive, Invert, Is,
    IsNot, keyword, Lambda, List, ListComp, Load, LShift, Lt, LtE, MatMult,
    Mod, mod, Module, Mult, Name, NameConstant, Nonlocal, Not, NotEq, NotIn,
    Num, operator, Or, Param, Pass, Pow, Raise, Return, RShift, Set, SetComp,
    Slice, slice, Starred, stmt, Store, Str, Sub, Subscript, Suite, Try, Tuple,
    UAdd, UnaryOp, unaryop, USub, While, With, withitem, Yield, YieldFrom
)

class NodeVisitor():
    __doc__ = ...  # type: str
    def visit(self, node: AST) -> Any: ...
    def generic_visit(self, node: AST) -> None: ...

class NodeTransformer(NodeVisitor):
    __doc__ = ...  # type: str
    def generic_visit(self, node: AST) -> None: ...

def parse(source: Union[str, bytes], filename: Union[str, bytes] = ..., mode: str = ...) -> AST: ...
def copy_location(new_node: AST, old_node: AST) -> AST: ...
def dump(node: AST, annotate_fields: bool = ..., include_attributes: bool = ...) -> str: ...
def fix_missing_locations(node: AST) -> AST: ...
def get_docstring(node: AST, clean: bool = ...) -> str: ...
def increment_lineno(node: AST, n: int = ...) -> AST: ...
def iter_child_nodes(node: AST) -> Iterator[AST]: ...
def iter_fields(node: AST) -> Iterator[typing.Tuple[str, Any]]: ...
def literal_eval(node_or_string: Union[str, AST]) -> Any: ...
def walk(node: AST) -> Iterator[AST]: ...

PyCF_ONLY_AST = ... # type: int




# Stubs for hashlib (Python 2)

from typing import Tuple

class _hash(object):
    # This is not actually in the module namespace.
    digest_size = 0
    block_size = 0
    def update(self, arg: str) -> None: ...
    def digest(self) -> str: ...
    def hexdigest(self) -> str: ...
    def copy(self) -> _hash: ...

def new(name: str, data: str = ...) -> _hash: ...

def md5(s: str = ...) -> _hash: ...
def sha1(s: str = ...) -> _hash: ...
def sha224(s: str = ...) -> _hash: ...
def sha256(s: str = ...) -> _hash: ...
def sha384(s: str = ...) -> _hash: ...
def sha512(s: str = ...) -> _hash: ...

algorithms = ...  # type: Tuple[str, ...]
algorithms_guaranteed = ...  # type: Tuple[str, ...]
algorithms_available = ...  # type: Tuple[str, ...]

def pbkdf2_hmac(name: str, password: str, salt: str, rounds: int, dklen: int = ...) -> str: ...



# Stubs for backports_abc (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

def mk_gen(): ...
def mk_awaitable(): ...
def mk_coroutine(): ...

Generator = ... # type: Any
Awaitable = ... # type: Any
Coroutine = ... # type: Any

def isawaitable(obj): ...

PATCHED = ... # type: Any

def patch(patch_inspect=True): ...



# Generated by stubtool 0.1, DO NOT EDIT
# See https://github.com/o11c/stubtool
#
# Stubs for six.moves.urllib.response (Python 2.7)

from urllib import addbase as addbase
from urllib import addclosehook as addclosehook
from urllib import addinfo as addinfo
from urllib import addinfourl as addinfourl



# Stubs for concurrent.futures.thread (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from . import _base

class _WorkItem:
    future = ... # type: Any
    fn = ... # type: Any
    args = ... # type: Any
    kwargs = ... # type: Any
    def __init__(self, future, fn, args, kwargs): ...
    def run(self): ...

class ThreadPoolExecutor(_base.Executor):
    def __init__(self, max_workers=None): ...
    def submit(self, fn, *args, **kwargs): ...
    def shutdown(self, wait=True): ...



# Stubs for requests.packages.urllib3.filepost (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from . import packages
#from .packages import six
from . import fields

#six = packages.six
#b = six.b
RequestField = fields.RequestField

writer = ...  # type: Any

def choose_boundary(): ...
def iter_field_objects(fields): ...
def iter_fields(fields): ...
def encode_multipart_formdata(fields, boundary=...): ...



# Stubs for sqlalchemy.sql.visitors (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class VisitableType(type):
    def __init__(cls, clsname, bases, clsdict) -> None: ...

class Visitable: ...

class ClauseVisitor:
    __traverse_options__ = ... # type: Any
    def traverse_single(self, obj, **kw): ...
    def iterate(self, obj): ...
    def traverse(self, obj): ...
    def chain(self, visitor): ...

class CloningVisitor(ClauseVisitor):
    def copy_and_process(self, list_): ...
    def traverse(self, obj): ...

class ReplacingCloningVisitor(CloningVisitor):
    def replace(self, elem): ...
    def traverse(self, obj): ...

def iterate(obj, opts): ...
def iterate_depthfirst(obj, opts): ...
def traverse_using(iterator, obj, visitors): ...
def traverse(obj, opts, visitors): ...
def traverse_depthfirst(obj, opts, visitors): ...
def cloned_traverse(obj, opts, visitors): ...
def replacement_traverse(obj, opts, replace): ...



# Stubs for contextlib

# NOTE: These are incomplete!

from typing import Callable, Generic, Iterator, TypeVar

_T = TypeVar('_T')

class ContextManager(Generic[_T]):
    def __enter__(self) -> _T: ...
    def __exit__(self, *exc_info) -> None: ...

# TODO this doesn't capture the relationship that the returned function's args are the same as func's.
def contextmanager(func: Callable[..., Iterator[_T]]) -> Callable[..., ContextManager[_T]]: ...

class closing(ContextManager[_T], Generic[_T]):
    def __init__(self, thing: _T) -> None: ...



# Stubs for yaml.serializer (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from yaml.error import YAMLError

class SerializerError(YAMLError): ...

class Serializer:
    ANCHOR_TEMPLATE = ... # type: Any
    use_encoding = ... # type: Any
    use_explicit_start = ... # type: Any
    use_explicit_end = ... # type: Any
    use_version = ... # type: Any
    use_tags = ... # type: Any
    serialized_nodes = ... # type: Any
    anchors = ... # type: Any
    last_anchor_id = ... # type: Any
    closed = ... # type: Any
    def __init__(self, encoding=..., explicit_start=..., explicit_end=..., version=..., tags=...) -> None: ...
    def open(self): ...
    def close(self): ...
    def serialize(self, node): ...
    def anchor_node(self, node): ...
    def generate_anchor(self, node): ...
    def serialize_node(self, node, parent, index): ...



# Stubs for time
# Ron Murawski <ron@horizonchess.com>

# based on: http://docs.python.org/3.2/library/time.html#module-time
# see: http://nullege.com/codes/search?cq=time

from typing import Tuple, Union

# ----- variables and constants -----
accept2dyear = False
altzone = 0
daylight = 0
timezone = 0
tzname = ... # type: Tuple[str, str]


# ----- classes/methods -----
class struct_time:
    # this is supposed to be a namedtuple object
    # namedtuple is not yet implemented (see file: mypy/stubs/collections.py)
    # see: http://docs.python.org/3.2/library/time.html#time.struct_time
    # see: http://nullege.com/codes/search/time.struct_time
    # TODO: namedtuple() object problem
    #namedtuple __init__(self, int, int, int, int, int, int, int, int, int):
    #    ...
    tm_year = 0
    tm_mon = 0
    tm_mday = 0
    tm_hour = 0
    tm_min = 0
    tm_sec = 0
    tm_wday = 0
    tm_yday = 0
    tm_isdst = 0


# ----- functions -----
def asctime(t: Union[Tuple[int, int, int, int, int, int, int, int, int],
                     struct_time,
                     None] = ...) -> str: ...  # return current time

def clock() -> float: ...

def ctime(secs: Union[float, None] = ...) -> str: ...  # return current time

def gmtime(secs: Union[float, None] = ...) -> struct_time: ...  # return current time

def localtime(secs: Union[float, None] = ...) -> struct_time: ...  # return current time

def mktime(t: Union[Tuple[int, int, int, int, int,
                          int, int, int, int],
                    struct_time]) -> float: ...

def sleep(secs: Union[int, float]) -> None: ...

def strftime(format: str, t: Union[Tuple[int, int, int, int, int,
                                         int, int, int, int],
                                   struct_time,
                                   None] = ...) -> str: ...  # return current time

def strptime(string: str,
             format: str = ...) -> struct_time: ...
def time() -> float: ...
def tzset() -> None: ...  # Unix only



# Stubs for http.cookies (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class CookieError(Exception): ...

class Morsel(dict):
    def __init__(self): ...
    @property
    def key(self): ...
    @key.setter
    def key(self, key): ...
    @property
    def value(self): ...
    @value.setter
    def value(self, value): ...
    @property
    def coded_value(self): ...
    @coded_value.setter
    def coded_value(self, coded_value): ...
    def __setitem__(self, K, V): ...
    def setdefault(self, key, val=None): ...
    def __eq__(self, morsel): ...
    __ne__ = ... # type: Any
    def copy(self): ...
    def update(self, values): ...
    def isReservedKey(self, K): ...
    def set(self, key, val, coded_val, LegalChars=...): ...
    def output(self, attrs=None, header=''): ...
    def js_output(self, attrs=None): ...
    def OutputString(self, attrs=None): ...

class BaseCookie(dict):
    def value_decode(self, val): ...
    def value_encode(self, val): ...
    def __init__(self, input=None): ...
    def __setitem__(self, key, value): ...
    def output(self, attrs=None, header='', sep=''): ...
    def js_output(self, attrs=None): ...
    def load(self, rawdata): ...

class SimpleCookie(BaseCookie):
    def value_decode(self, val): ...
    def value_encode(self, val): ...



# Stubs for six (Python 2.7)

from __future__ import print_function

from typing import (
    Any, AnyStr, Callable, Iterable, Mapping, Optional,
    Pattern, Tuple, TypeVar, Union, overload,
)
import typing

import unittest
import types

_T = TypeVar('_T')
_K = TypeVar('_K')
_V = TypeVar('_V')

# TODO make constant, then move this stub to 2and3
# https://github.com/python/typeshed/issues/17
PY2 = True
PY3 = False
PY34 = False

string_types = basestring,
integer_types = (int, long)
class_types = (type, types.ClassType)
text_type = unicode
binary_type = str

MAXSIZE = ... # type: int

#def add_move
#def remove_move

def advance_iterator(it: typing.Iterator[_T]) -> _T: ...
next = advance_iterator

def callable(obj: object) -> bool: ...

def get_unbound_function(unbound: types.MethodType) -> types.FunctionType: ...
def create_bound_method(func: types.FunctionType, obj: object) -> types.MethodType: ...
def create_unbound_method(func: types.FunctionType, cls: Union[type, types.ClassType]) -> types.MethodType: ...

class Iterator:
    def next(self) -> Any: ...

def get_method_function(meth: types.MethodType) -> types.FunctionType: ...
def get_method_self(meth: types.MethodType) -> Optional[object]: ...
def get_function_closure(fun: types.FunctionType) -> Optional[Tuple[types._Cell, ...]]: ...
def get_function_code(fun: types.FunctionType) -> types.CodeType: ...
def get_function_defaults(fun: types.FunctionType) -> Optional[Tuple[Any, ...]]: ...
def get_function_globals(fun: types.FunctionType) -> Dict[str, Any]: ...

def iterkeys(d: Mapping[_K, _V]) -> typing.Iterator[_K]: ...
def itervalues(d: Mapping[_K, _V]) -> typing.Iterator[_V]: ...
def iteritems(d: Mapping[_K, _V]) -> typing.Iterator[Tuple[_K, _V]]: ...
#def iterlists

# TODO fix return types - python2 typing doesn't include KeysView etc yet.
def viewkeys(d: Mapping[_K, _V]) -> Iterable[_K]: ...
def viewvalues(d: Mapping[_K, _V]) -> Iterable[_V]: ...
def viewitems(d: Mapping[_K, _V]) -> Iterable[Tuple[_K, _V]]: ...

def b(s: str) -> binary_type: ...
def u(s: str) -> text_type: ...
from __builtin__ import unichr as unichr
int2byte = chr
def byte2int(bs: binary_type) -> int: ...
def indexbytes(buf: binary_type, i: int) -> int: ...
def iterbytes(buf: binary_type) -> typing.Iterator[int]: ...
from StringIO import StringIO as StringIO, StringIO as BytesIO

def assertCountEqual(self: unittest.TestCase, first: Iterable[_T], second: Iterable[_T], msg: str = ...) -> None: ...
@overload
def assertRaisesRegex(self: unittest.TestCase, msg: str = ...) -> Any: ...
@overload
def assertRaisesRegex(self: unittest.TestCase, callable_obj: Callable[..., Any], *args: Any, **kwargs: Any) -> Any: ...
def assertRegex(self: unittest.TestCase, text: AnyStr, expected_regex: Union[AnyStr, Pattern[AnyStr]], msg: str = ...) -> None: ...

def reraise(tp: type, value: Optional[BaseException], tb: types.TracebackType = ...) -> None: ...
def exec_(_code_: Union[unicode, types.CodeType], _globs_: Dict[str, Any] = ..., _locs_: Dict[str, Any] = ...): ...
def raise_from(value: BaseException, from_value: BaseException) -> None: ...

print_ = print

from functools import wraps as wraps

def with_metaclass(meta: type, *bases: type) -> type: ...
def add_metaclass(metaclass: type) -> Callable[[_T], _T]: ...
def python_2_unicode_compatible(klass: _T) -> _T: ...



# Stubs for thrift.transport (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Names in __all__ with no definition:
#   THttpClient
#   TSocket
#   TTransport
#   TZlibTransport



# Generated by stubtool 0.1, DO NOT EDIT
# See https://github.com/o11c/stubtool
#
# Stubs for six.moves.urllib.response (Python 3.2)

from urllib.response import addbase as addbase
from urllib.response import addclosehook as addclosehook
from urllib.response import addinfo as addinfo
from urllib.response import addinfourl as addinfourl



from typing import AnyStr

__all__ = ... # type: str

from asyncio import transports

class BaseProtocol:
    def connection_made(self, transport: transports.BaseTransport) -> None: ...
    def connection_lost(self, exc: Exception) -> None: ...
    def pause_writing(self) -> None: ...
    def resume_writing(self) -> None: ...

class Protocol(BaseProtocol):
    def data_received(self, data: AnyStr) -> None: ...
    def eof_received(self) -> bool: ...

class DatagramProtocol(BaseProtocol):
    def datagram_received(self, data: AnyStr, addr: str) -> None: ...
    def error_received(self, exc: Exception) -> None: ...

class SubprocessProtocol(BaseProtocol):
    def pipe_data_received(self, fd: int, data: AnyStr) -> None: ...
    def pipe_connection_lost(self, fd: int, exc: Exception) -> None: ...
    def process_exited(self) -> None: ...



"""Type inference constraints."""

from typing import List, Optional, cast

from mypy.types import (
    CallableType, Type, TypeVisitor, UnboundType, AnyType, Void, NoneTyp, TypeVarType,
    Instance, TupleType, UnionType, Overloaded, ErasedType, PartialType, DeletedType,
    is_named_instance
)
from mypy.maptype import map_instance_to_supertype
from mypy import nodes
import mypy.subtypes
from mypy.erasetype import erase_typevars


SUBTYPE_OF = 0  # type: int
SUPERTYPE_OF = 1  # type: int


class Constraint:
    """A representation of a type constraint.

    It can be either T <: type or T :> type (T is a type variable).
    """

    type_var = 0   # Type variable id
    op = 0         # SUBTYPE_OF or SUPERTYPE_OF
    target = None  # type: Type

    def __init__(self, type_var: int, op: int, target: Type) -> None:
        self.type_var = type_var
        self.op = op
        self.target = target

    def __repr__(self) -> str:
        op_str = '<:'
        if self.op == SUPERTYPE_OF:
            op_str = ':>'
        return '{} {} {}'.format(self.type_var, op_str, self.target)


def infer_constraints_for_callable(
        callee: CallableType, arg_types: List[Optional[Type]], arg_kinds: List[int],
        formal_to_actual: List[List[int]]) -> List[Constraint]:
    """Infer type variable constraints for a callable and actual arguments.

    Return a list of constraints.
    """
    constraints = []  # type: List[Constraint]
    tuple_counter = [0]

    for i, actuals in enumerate(formal_to_actual):
        for actual in actuals:
            if arg_types[actual] is None:
                continue

            actual_type = get_actual_type(arg_types[actual], arg_kinds[actual],
                                          tuple_counter)
            c = infer_constraints(callee.arg_types[i], actual_type,
                                  SUPERTYPE_OF)
            constraints.extend(c)

    return constraints


def get_actual_type(arg_type: Type, kind: int,
                    tuple_counter: List[int]) -> Type:
    """Return the type of an actual argument with the given kind.

    If the argument is a *arg, return the individual argument item.
    """

    if kind == nodes.ARG_STAR:
        if isinstance(arg_type, Instance):
            if arg_type.type.fullname() == 'builtins.list':
                # List *arg.
                return arg_type.args[0]
            elif arg_type.args:
                # TODO try to map type arguments to Iterable
                return arg_type.args[0]
            else:
                return AnyType()
        elif isinstance(arg_type, TupleType):
            # Get the next tuple item of a tuple *arg.
            tuple_counter[0] += 1
            return arg_type.items[tuple_counter[0] - 1]
        else:
            return AnyType()
    elif kind == nodes.ARG_STAR2:
        if isinstance(arg_type, Instance) and (arg_type.type.fullname() == 'builtins.dict'):
            # Dict **arg. TODO more general (Mapping)
            return arg_type.args[1]
        else:
            return AnyType()
    else:
        # No translation for other kinds.
        return arg_type


def infer_constraints(template: Type, actual: Type,
                      direction: int) -> List[Constraint]:
    """Infer type constraints.

    Match a template type, which may contain type variable references,
    recursively against a type which does not contain (the same) type
    variable references. The result is a list of type constrains of
    form 'T is a supertype/subtype of x', where T is a type variable
    present in the template and x is a type without reference to type
    variables present in the template.

    Assume T and S are type variables. Now the following results can be
    calculated (read as '(template, actual) --> result'):

      (T, X)            -->  T :> X
      (X[T], X[Y])      -->  T <: Y and T :> Y
      ((T, T), (X, Y))  -->  T :> X and T :> Y
      ((T, S), (X, Y))  -->  T :> X and S :> Y
      (X[T], Any)       -->  T <: Any and T :> Any

    The constraints are represented as Constraint objects.
    """

    # If the template is simply a type variable, emit a Constraint directly.
    # We need to handle this case before handling Unions for two reasons:
    #  1. "T <: Union[U1, U2]" is not equivalent to "T <: U1 or T <: U2",
    #     because T can itself be a union (notably, Union[U1, U2] itself).
    #  2. "T :> Union[U1, U2]" is logically equivalent to "T :> U1 and
    #     T :> U2", but they are not equivalent to the constraint solver,
    #     which never introduces new Union types (it uses join() instead).
    if isinstance(template, TypeVarType):
        return [Constraint(template.id, direction, actual)]

    # Now handle the case of either template or actual being a Union.
    # For a Union to be a subtype of another type, every item of the Union
    # must be a subtype of that type, so concatenate the constraints.
    if direction == SUBTYPE_OF and isinstance(template, UnionType):
        res = []
        for t_item in template.items:
            res.extend(infer_constraints(t_item, actual, direction))
        return res
    if direction == SUPERTYPE_OF and isinstance(actual, UnionType):
        res = []
        for a_item in actual.items:
            res.extend(infer_constraints(template, a_item, direction))
        return res

    # Now the potential subtype is known not to be a Union or a type
    # variable that we are solving for. In that case, for a Union to
    # be a supertype of the potential subtype, some item of the Union
    # must be a supertype of it.
    if direction == SUBTYPE_OF and isinstance(actual, UnionType):
        return any_constraints(
            [infer_constraints_if_possible(template, a_item, direction)
             for a_item in actual.items])
    if direction == SUPERTYPE_OF and isinstance(template, UnionType):
        return any_constraints(
            [infer_constraints_if_possible(t_item, actual, direction)
             for t_item in template.items])

    # Remaining cases are handled by ConstraintBuilderVisitor.
    return template.accept(ConstraintBuilderVisitor(actual, direction))


def infer_constraints_if_possible(template: Type, actual: Type,
                                  direction: int) -> Optional[List[Constraint]]:
    """Like infer_constraints, but return None if the input relation is
    known to be unsatisfiable, for example if template=List[T] and actual=int.
    (In this case infer_constraints would return [], just like it would for
    an automatically satisfied relation like template=List[T] and actual=object.)
    """
    if (direction == SUBTYPE_OF and
            not mypy.subtypes.is_subtype(erase_typevars(template), actual)):
        return None
    if (direction == SUPERTYPE_OF and
            not mypy.subtypes.is_subtype(actual, erase_typevars(template))):
        return None
    return infer_constraints(template, actual, direction)


def any_constraints(options: List[Optional[List[Constraint]]]) -> List[Constraint]:
    """Deduce what we can from a collection of constraint lists given that
    at least one of the lists must be satisfied. A None element in the
    list of options represents an unsatisfiable constraint and is ignored.
    """
    valid_options = [option for option in options if option is not None]
    if len(valid_options) == 1:
        return valid_options[0]
    # Otherwise, there are either no valid options or multiple valid options.
    # Give up and deduce nothing.
    return []

    # TODO: In the latter case, it could happen that every valid
    # option requires the same constraint on the same variable. Then
    # we could include that that constraint in the result. Or more
    # generally, if a given (variable, direction) pair appears in
    # every option, combine the bounds with meet/join.


class ConstraintBuilderVisitor(TypeVisitor[List[Constraint]]):
    """Visitor class for inferring type constraints."""

    # The type that is compared against a template
    # TODO: The value may be None. Is that actually correct?
    actual = None  # type: Type

    def __init__(self, actual: Type, direction: int) -> None:
        # Direction must be SUBTYPE_OF or SUPERTYPE_OF.
        self.actual = actual
        self.direction = direction

    # Trivial leaf types

    def visit_unbound_type(self, template: UnboundType) -> List[Constraint]:
        return []

    def visit_any(self, template: AnyType) -> List[Constraint]:
        return []

    def visit_void(self, template: Void) -> List[Constraint]:
        return []

    def visit_none_type(self, template: NoneTyp) -> List[Constraint]:
        return []

    def visit_erased_type(self, template: ErasedType) -> List[Constraint]:
        return []

    def visit_deleted_type(self, template: DeletedType) -> List[Constraint]:
        return []

    # Errors

    def visit_partial_type(self, template: PartialType) -> List[Constraint]:
        # We can't do anything useful with a partial type here.
        assert False, "Internal error"

    # Non-trivial leaf type

    def visit_type_var(self, template: TypeVarType) -> List[Constraint]:
        assert False, ("Unexpected TypeVarType in ConstraintBuilderVisitor"
                       " (should have been handled in infer_constraints)")

    # Non-leaf types

    def visit_instance(self, template: Instance) -> List[Constraint]:
        actual = self.actual
        res = []  # type: List[Constraint]
        if isinstance(actual, Instance):
            instance = actual
            if (self.direction == SUBTYPE_OF and
                    template.type.has_base(instance.type.fullname())):
                mapped = map_instance_to_supertype(template, instance.type)
                for i in range(len(instance.args)):
                    # The constraints for generic type parameters are
                    # invariant. Include constraints from both directions
                    # to achieve the effect.
                    res.extend(infer_constraints(
                        mapped.args[i], instance.args[i], self.direction))
                    res.extend(infer_constraints(
                        mapped.args[i], instance.args[i], neg_op(self.direction)))
                return res
            elif (self.direction == SUPERTYPE_OF and
                    instance.type.has_base(template.type.fullname())):
                mapped = map_instance_to_supertype(instance, template.type)
                for j in range(len(template.args)):
                    # The constraints for generic type parameters are
                    # invariant.
                    res.extend(infer_constraints(
                        template.args[j], mapped.args[j], self.direction))
                    res.extend(infer_constraints(
                        template.args[j], mapped.args[j], neg_op(self.direction)))
                return res
        if isinstance(actual, AnyType):
            # IDEA: Include both ways, i.e. add negation as well?
            return self.infer_against_any(template.args)
        if (isinstance(actual, TupleType) and
            (is_named_instance(template, 'typing.Iterable') or
             is_named_instance(template, 'typing.Sequence') or
             is_named_instance(template, 'typing.Reversible'))
                and self.direction == SUPERTYPE_OF):
            for item in actual.items:
                cb = infer_constraints(template.args[0], item, SUPERTYPE_OF)
                res.extend(cb)
            return res
        else:
            return []

    def visit_callable_type(self, template: CallableType) -> List[Constraint]:
        if isinstance(self.actual, CallableType):
            cactual = self.actual
            # FIX verify argument counts
            # FIX what if one of the functions is generic
            res = []  # type: List[Constraint]

            # We can't infer constraints from arguments if the template is Callable[..., T] (with
            # literal '...').
            if not template.is_ellipsis_args:
                # The lengths should match, but don't crash (it will error elsewhere).
                for t, a in zip(template.arg_types, cactual.arg_types):
                    # Negate direction due to function argument type contravariance.
                    res.extend(infer_constraints(t, a, neg_op(self.direction)))
            res.extend(infer_constraints(template.ret_type, cactual.ret_type,
                                         self.direction))
            return res
        elif isinstance(self.actual, AnyType):
            # FIX what if generic
            res = self.infer_against_any(template.arg_types)
            res.extend(infer_constraints(template.ret_type, AnyType(),
                                         self.direction))
            return res
        elif isinstance(self.actual, Overloaded):
            return self.infer_against_overloaded(self.actual, template)
        else:
            return []

    def infer_against_overloaded(self, overloaded: Overloaded,
                                 template: CallableType) -> List[Constraint]:
        # Create constraints by matching an overloaded type against a template.
        # This is tricky to do in general. We cheat by only matching against
        # the first overload item, and by only matching the return type. This
        # seems to work somewhat well, but we should really use a more
        # reliable technique.
        item = find_matching_overload_item(overloaded, template)
        return infer_constraints(template.ret_type, item.ret_type,
                                 self.direction)

    def visit_tuple_type(self, template: TupleType) -> List[Constraint]:
        actual = self.actual
        if isinstance(actual, TupleType) and len(actual.items) == len(template.items):
            res = []  # type: List[Constraint]
            for i in range(len(template.items)):
                res.extend(infer_constraints(template.items[i],
                                             actual.items[i],
                                             self.direction))
            return res
        elif isinstance(actual, AnyType):
            return self.infer_against_any(template.items)
        else:
            return []

    def visit_union_type(self, template: UnionType) -> List[Constraint]:
        assert False, ("Unexpected UnionType in ConstraintBuilderVisitor"
                       " (should have been handled in infer_constraints)")

    def infer_against_any(self, types: List[Type]) -> List[Constraint]:
        res = []  # type: List[Constraint]
        for t in types:
            res.extend(infer_constraints(t, AnyType(), self.direction))
        return res

    def visit_overloaded(self, type: Overloaded) -> List[Constraint]:
        res = []  # type: List[Constraint]
        for t in type.items():
            res.extend(infer_constraints(t, self.actual, self.direction))
        return res


def neg_op(op: int) -> int:
    """Map SubtypeOf to SupertypeOf and vice versa."""

    if op == SUBTYPE_OF:
        return SUPERTYPE_OF
    elif op == SUPERTYPE_OF:
        return SUBTYPE_OF
    else:
        raise ValueError('Invalid operator {}'.format(op))


def find_matching_overload_item(overloaded: Overloaded, template: CallableType) -> CallableType:
    """Disambiguate overload item against a template."""
    items = overloaded.items()
    for item in items:
        # Return type may be indeterminate in the template, so ignore it when performing a
        # subtype check.
        if mypy.subtypes.is_callable_subtype(item, template, ignore_return=True):
            return item
    # Fall back to the first item if we can't find a match. This is totally arbitrary --
    # maybe we should just bail out at this point.
    return items[0]



"""Utilities for calculating and reporting statistics about types."""

import cgi
import os.path
import re

from typing import Any, Dict, List, cast, Tuple

from mypy.traverser import TraverserVisitor
from mypy.types import (
    Type, AnyType, Instance, FunctionLike, TupleType, Void, TypeVarType,
    TypeQuery, ANY_TYPE_STRATEGY, CallableType
)
from mypy import nodes
from mypy.nodes import (
    Node, FuncDef, TypeApplication, AssignmentStmt, NameExpr, CallExpr,
    MemberExpr, OpExpr, ComparisonExpr, IndexExpr, UnaryExpr, YieldFromExpr
)


TYPE_EMPTY = 0
TYPE_PRECISE = 1
TYPE_IMPRECISE = 2
TYPE_ANY = 3

precision_names = [
    'empty',
    'precise',
    'imprecise',
    'any',
]


class StatisticsVisitor(TraverserVisitor):
    def __init__(self, inferred: bool, typemap: Dict[Node, Type] = None,
                 all_nodes: bool = False) -> None:
        self.inferred = inferred
        self.typemap = typemap
        self.all_nodes = all_nodes

        self.num_precise = 0
        self.num_imprecise = 0
        self.num_any = 0

        self.num_simple = 0
        self.num_generic = 0
        self.num_tuple = 0
        self.num_function = 0
        self.num_typevar = 0
        self.num_complex = 0

        self.line = -1

        self.line_map = {}  # type: Dict[int, int]

        self.output = []  # type: List[str]

        TraverserVisitor.__init__(self)

    def visit_func_def(self, o: FuncDef) -> None:
        self.line = o.line
        if len(o.expanded) > 1:
            if o in o.expanded:
                print('ERROR: cycle in function expansion; skipping')
                return
            for defn in o.expanded:
                self.visit_func_def(cast(FuncDef, defn))
        else:
            if o.type:
                sig = cast(CallableType, o.type)
                arg_types = sig.arg_types
                if (sig.arg_names and sig.arg_names[0] == 'self' and
                        not self.inferred):
                    arg_types = arg_types[1:]
                for arg in arg_types:
                    self.type(arg)
                self.type(sig.ret_type)
            elif self.all_nodes:
                self.record_line(self.line, TYPE_ANY)
            super().visit_func_def(o)

    def visit_type_application(self, o: TypeApplication) -> None:
        self.line = o.line
        for t in o.types:
            self.type(t)
        super().visit_type_application(o)

    def visit_assignment_stmt(self, o: AssignmentStmt) -> None:
        self.line = o.line
        if (isinstance(o.rvalue, nodes.CallExpr) and
            isinstance(cast(nodes.CallExpr, o.rvalue).analyzed,
                       nodes.TypeVarExpr)):
            # Type variable definition -- not a real assignment.
            return
        if o.type:
            self.type(o.type)
        elif self.inferred:
            for lvalue in o.lvalues:
                if isinstance(lvalue, nodes.TupleExpr):
                    items = lvalue.items
                elif isinstance(lvalue, nodes.ListExpr):
                    items = lvalue.items
                else:
                    items = [lvalue]
                for item in items:
                    if hasattr(item, 'is_def') and cast(Any, item).is_def:
                        t = self.typemap.get(item)
                        if t:
                            self.type(t)
                        else:
                            self.log('  !! No inferred type on line %d' %
                                     self.line)
                            self.record_line(self.line, TYPE_ANY)
        super().visit_assignment_stmt(o)

    def visit_name_expr(self, o: NameExpr) -> None:
        self.process_node(o)
        super().visit_name_expr(o)

    def visit_yield_from_expr(self, o: YieldFromExpr) -> None:
        if o.expr:
            o.expr.accept(self)

    def visit_call_expr(self, o: CallExpr) -> None:
        self.process_node(o)
        if o.analyzed:
            o.analyzed.accept(self)
        else:
            o.callee.accept(self)
            for a in o.args:
                a.accept(self)

    def visit_member_expr(self, o: MemberExpr) -> None:
        self.process_node(o)
        super().visit_member_expr(o)

    def visit_op_expr(self, o: OpExpr) -> None:
        self.process_node(o)
        super().visit_op_expr(o)

    def visit_comparison_expr(self, o: ComparisonExpr) -> None:
        self.process_node(o)
        super().visit_comparison_expr(o)

    def visit_index_expr(self, o: IndexExpr) -> None:
        self.process_node(o)
        super().visit_index_expr(o)

    def visit_unary_expr(self, o: UnaryExpr) -> None:
        self.process_node(o)
        super().visit_unary_expr(o)

    def process_node(self, node: Node) -> None:
        if self.all_nodes:
            typ = self.typemap.get(node)
            if typ:
                self.line = node.line
                self.type(typ)

    def type(self, t: Type) -> None:
        if isinstance(t, AnyType):
            self.log('  !! Any type around line %d' % self.line)
            self.num_any += 1
            self.record_line(self.line, TYPE_ANY)
        elif ((not self.all_nodes and is_imprecise(t)) or
              (self.all_nodes and is_imprecise2(t))):
            self.log('  !! Imprecise type around line %d' % self.line)
            self.num_imprecise += 1
            self.record_line(self.line, TYPE_IMPRECISE)
        else:
            self.num_precise += 1
            self.record_line(self.line, TYPE_PRECISE)

        if isinstance(t, Instance):
            if t.args:
                if any(is_complex(arg) for arg in t.args):
                    self.num_complex += 1
                else:
                    self.num_generic += 1
            else:
                self.num_simple += 1
        elif isinstance(t, Void):
            self.num_simple += 1
        elif isinstance(t, FunctionLike):
            self.num_function += 1
        elif isinstance(t, TupleType):
            if any(is_complex(item) for item in t.items):
                self.num_complex += 1
            else:
                self.num_tuple += 1
        elif isinstance(t, TypeVarType):
            self.num_typevar += 1

    def log(self, string: str) -> None:
        self.output.append(string)

    def record_line(self, line: int, precision: int) -> None:
        self.line_map[line] = max(precision,
                                  self.line_map.get(line, TYPE_PRECISE))


def dump_type_stats(tree: Node, path: str, inferred: bool = False,
                    typemap: Dict[Node, Type] = None) -> None:
    if is_special_module(path):
        return
    print(path)
    visitor = StatisticsVisitor(inferred, typemap)
    tree.accept(visitor)
    for line in visitor.output:
        print(line)
    print('  ** precision **')
    print('  precise  ', visitor.num_precise)
    print('  imprecise', visitor.num_imprecise)
    print('  any      ', visitor.num_any)
    print('  ** kinds **')
    print('  simple   ', visitor.num_simple)
    print('  generic  ', visitor.num_generic)
    print('  function ', visitor.num_function)
    print('  tuple    ', visitor.num_tuple)
    print('  TypeVar  ', visitor.num_typevar)
    print('  complex  ', visitor.num_complex)
    print('  any      ', visitor.num_any)


def is_special_module(path: str) -> bool:
    return os.path.basename(path) in ('abc.py', 'typing.py', 'builtins.py')


def is_imprecise(t: Type) -> bool:
    return t.accept(HasAnyQuery())


class HasAnyQuery(TypeQuery):
    def __init__(self) -> None:
        super().__init__(False, ANY_TYPE_STRATEGY)

    def visit_any(self, t: AnyType) -> bool:
        return True

    def visit_instance(self, t: Instance) -> bool:
        if t.type.fullname() == 'builtins.tuple':
            return True
        else:
            return super().visit_instance(t)


def is_imprecise2(t: Type) -> bool:
    return t.accept(HasAnyQuery2())


class HasAnyQuery2(HasAnyQuery):
    def visit_callable_type(self, t: CallableType) -> bool:
        # We don't want to flag references to functions with some Any
        # argument types (etc.) since they generally don't mean trouble.
        return False


def is_generic(t: Type) -> bool:
    return isinstance(t, Instance) and bool(cast(Instance, t).args)


def is_complex(t: Type) -> bool:
    return is_generic(t) or isinstance(t, (FunctionLike, TupleType,
                                           TypeVarType))


html_files = []  # type: List[Tuple[str, str, int, int]]


def generate_html_report(tree: Node, path: str, type_map: Dict[Node, Type],
                         output_dir: str) -> None:
    if is_special_module(path):
        return
    # There may be more than one right answer for "what should we do here?"
    # but this is a reasonable one.
    path = os.path.relpath(path)
    if path.startswith('..'):
        return
    visitor = StatisticsVisitor(inferred=True, typemap=type_map, all_nodes=True)
    tree.accept(visitor)
    assert not os.path.isabs(path) and not path.startswith('..')
    # This line is *wrong* if the preceding assert fails.
    target_path = os.path.join(output_dir, 'html', path)
    # replace .py or .pyi with .html
    target_path = os.path.splitext(target_path)[0] + '.html'
    assert target_path.endswith('.html')
    ensure_dir_exists(os.path.dirname(target_path))
    output = []  # type: List[str]
    append = output.append
    append('''\
<html>
<head>
  <style>
    .red { background-color: #faa; }
    .yellow { background-color: #ffa; }
    .white { }
    .lineno { color: #999; }
  </style>
</head>
<body>
<pre>''')
    num_imprecise_lines = 0
    num_lines = 0
    with open(path) as input_file:
        for i, line in enumerate(input_file):
            lineno = i + 1
            status = visitor.line_map.get(lineno, TYPE_PRECISE)
            style_map = {TYPE_PRECISE: 'white',
                         TYPE_IMPRECISE: 'yellow',
                         TYPE_ANY: 'red'}
            style = style_map[status]
            append('<span class="lineno">%4d</span>   ' % lineno +
                   '<span class="%s">%s</span>' % (style,
                                                   cgi.escape(line)))
            if status != TYPE_PRECISE:
                num_imprecise_lines += 1
            if line.strip():
                num_lines += 1
    append('</pre>')
    append('</body></html>')
    with open(target_path, 'w') as output_file:
        output_file.writelines(output)
    target_path = target_path[len(output_dir) + 1:]
    html_files.append((path, target_path, num_lines, num_imprecise_lines))


def generate_html_index(output_dir: str) -> None:
    path = os.path.join(output_dir, 'index.html')
    output = []  # type: List[str]
    append = output.append
    append('''\
<html>
<head>
  <style>
  body { font-family: courier; }
  table { border-collapse: collapse; }
  table tr td { border: 1px solid black; }
  td { padding: 0.4em; }
  .red { background-color: #faa; }
  .yellow { background-color: #ffa; }
  </style>
</head>
<body>''')
    append('<h1>Mypy Type Check Coverage Report</h1>\n')
    append('<table>\n')
    for source_path, target_path, num_lines, num_imprecise in sorted(html_files):
        if num_lines == 0:
            continue
        source_path = os.path.normpath(source_path)
        # TODO: Windows paths.
        if (source_path.startswith('stubs/') or
                '/stubs/' in source_path):
            continue
        percent = 100.0 * num_imprecise / num_lines
        style = ''
        if percent >= 20:
            style = 'class="red"'
        elif percent >= 5:
            style = 'class="yellow"'
        append('<tr %s><td><a href="%s">%s</a><td>%.1f%% imprecise<td>%d LOC\n' % (
            style, target_path, source_path, percent, num_lines))
    append('</table>\n')
    append('</body></html>')
    with open(path, 'w') as file:
        file.writelines(output)
    print('Generated HTML report (old): %s' % os.path.abspath(path))


def ensure_dir_exists(dir: str) -> None:
    if not os.path.exists(dir):
        os.makedirs(dir)



# Stubs for distutils.version (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class Version:
    def __init__(self, vstring=...) -> None: ...

class StrictVersion(Version):
    version_re = ... # type: Any
    version = ... # type: Any
    prerelease = ... # type: Any
    def parse(self, vstring): ...
    def __cmp__(self, other): ...

class LooseVersion(Version):
    component_re = ... # type: Any
    def __init__(self, vstring=...) -> None: ...
    vstring = ... # type: Any
    version = ... # type: Any
    def parse(self, vstring): ...
    def __cmp__(self, other): ...



# Stubs for sqlalchemy.pool (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from . import log
from . import util

threading = util.threading
memoized_property = util.memoized_property
chop_traceback = util.chop_traceback

proxies = ... # type: Any

def manage(module, **params): ...
def clear_managers(): ...

reset_rollback = ... # type: Any
reset_commit = ... # type: Any
reset_none = ... # type: Any

class _ConnDialect:
    def do_rollback(self, dbapi_connection): ...
    def do_commit(self, dbapi_connection): ...
    def do_close(self, dbapi_connection): ...

class Pool(log.Identified):
    logging_name = ... # type: Any
    echo = ... # type: Any
    def __init__(self, creator, recycle=..., echo=..., use_threadlocal=..., logging_name=..., reset_on_return=..., listeners=..., events=..., _dispatch=..., _dialect=...) -> None: ...
    def add_listener(self, listener): ...
    def unique_connection(self): ...
    def recreate(self): ...
    def dispose(self): ...
    def connect(self): ...
    def status(self): ...

    _threadconns = ... # type: Any
    _creator = ... # type: Any
    _recycle = ... # type: Any
    _invalidate_time = ... # type: Any
    dispatch = ... # type: Any
    _dialect = ... # type: Any
    _orig_logging_name = ... # type: Any
    _reset_on_return = ... # type: Any
    _use_threadlocal = ... # type: Any

class _ConnectionRecord:
    connection = ... # type: Any
    finalize_callback = ... # type: Any
    def __init__(self, pool) -> None: ...
    def info(self): ...
    @classmethod
    def checkout(cls, pool): ...
    fairy_ref = ... # type: Any
    def checkin(self): ...
    def close(self): ...
    def invalidate(self, e=..., soft=...): ...
    def get_connection(self): ...

class _ConnectionFairy:
    connection = ... # type: Any
    def __init__(self, dbapi_connection, connection_record, echo) -> None: ...
    @property
    def is_valid(self): ...
    def info(self): ...
    def invalidate(self, e=..., soft=...): ...
    def cursor(self, *args, **kwargs): ...
    def __getattr__(self, key): ...
    def detach(self): ...
    def close(self): ...

class SingletonThreadPool(Pool):
    size = ... # type: Any
    def __init__(self, creator, pool_size=..., **kw) -> None: ...
    def recreate(self): ...
    def dispose(self): ...
    def status(self): ...

class QueuePool(Pool):
    def __init__(self, creator, pool_size=..., max_overflow=..., timeout=..., **kw) -> None: ...
    def recreate(self): ...
    def dispose(self): ...
    def status(self): ...
    def size(self): ...
    def checkedin(self): ...
    def overflow(self): ...
    def checkedout(self): ...

class NullPool(Pool):
    def status(self): ...
    def recreate(self): ...
    def dispose(self): ...

class StaticPool(Pool):
    def connection(self): ...
    def status(self): ...
    def dispose(self): ...
    def recreate(self): ...

class AssertionPool(Pool):
    def __init__(self, *args, **kw) -> None: ...
    def status(self): ...
    def dispose(self): ...
    def recreate(self): ...

class _DBProxy:
    module = ... # type: Any
    kw = ... # type: Any
    poolclass = ... # type: Any
    pools = ... # type: Any
    def __init__(self, module, poolclass=..., **kw) -> None: ...
    def close(self): ...
    def __del__(self): ...
    def __getattr__(self, key): ...
    def get_pool(self, *args, **kw): ...
    def connect(self, *args, **kw): ...
    def dispose(self, *args, **kw): ...



from typing import Any, Iterable

from .visitors import ClauseVisitor
from .. import util

class Immutable(object):
    def unique_params(self, *optionaldict, **kwargs): ...
    def params(self, *optionaldict, **kwargs): ...
    def _clone(self) -> Immutable: ...

class DialectKWArgs(object):
    def argument_for(cls, dialect_name, argument_name, default): ...
    def kwargs(self): ...
    def dialect_options(self): ...

class Generative(object): ...

class Executable(Generative):
    def execution_options(self, **kw): ...
    def execute(self, *multiparams, **params): ...
    def scalar(self, *multiparams, **params): ...

    @property
    def bind(self): ...

class SchemaEventTarget(object): ...
class SchemaVisitor(ClauseVisitor): ...
class ColumnCollection(util.OrderedProperties):
    def replace(self, column): ...
    def add(self, column): ...
    def clear(self): ...
    def remove(self, column): ...
    def update(self, iter: Iterable[Any]): ...
    def extend(self, iter: Iterable[Any]): ...
    def contains_column(self, col): ...
    def as_immutable(self): ...

class ImmutableColumnCollection(util.ImmutableProperties, ColumnCollection): ...

class ColumnSet(util.ordered_column_set): ...

def _bind_or_error(schemaitem, msg): ...



# Stubs for glob

# Based on http://docs.python.org/3.2/library/glob.html

from typing import List, Iterator, AnyStr

def glob(pathname: AnyStr) -> List[AnyStr]: ...
def iglob(pathname: AnyStr) -> Iterator[AnyStr]: ...



# Stubs for selenium.webdriver.remote.webdriver (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Dict, List
from .mobile import Mobile as Mobile
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.remote.errorhandler import ErrorHandler
from selenium.webdriver.remote.file_detector import FileDetector

Capabilities = Dict[basestring, Any]
ExecuteResult = Dict[basestring, Any]  # containing 'success', 'value', 'sessionId'

class WebDriver:
    command_executor = ... # type: basestring
    session_id = ... # type: Any
    capabilities = ... # type: Capabilities
    error_handler = ... # type: ErrorHandler
    file_detector = ... # type: FileDetector
    def __init__(self,
                 command_executor: basestring='',
                 desired_capabilities: Capabilities=None,
                 browser_profile=None,
                 proxy=None,
                 keep_alive:bool=False
                 ) -> None: ...
    @property
    def mobile(self) -> Mobile: ...
    @property
    def name(self) -> basestring: ...
    def start_client(self): ...
    def stop_client(self): ...
    w3c = ... # type: Any
    def start_session(self, desired_capabilities, browser_profile=None): ...
    def create_web_element(self, element_id: basestring) -> WebElement: ...
    def execute(self, driver_command: basestring, params: Optional[Dict[basestring, Any]]=None) -> ExecuteResult: ...
    def get(self, url: basestring) -> None: ...
    @property
    def title(self) -> basestring: ...
    def find_element_by_id(self, id_: basestring) -> WebElement: ...
    def find_elements_by_id(self, id_: basestring) -> List[WebElement]: ...
    def find_element_by_xpath(self, xpath: basestring) -> WebElement: ...
    def find_elements_by_xpath(self, xpath: basestring) -> List[WebElement]: ...
    def find_element_by_link_text(self, link_text: basestring) -> WebElement: ...
    def find_elements_by_link_text(self, text: basestring) -> List[WebElement]: ...
    def find_element_by_partial_link_text(self, link_text: basestring) -> WebElement: ...
    def find_elements_by_partial_link_text(self, link_text: basestring) -> List[WebElement]: ...
    def find_element_by_name(self, name: basestring) -> WebElement: ...
    def find_elements_by_name(self, name: basestring) -> List[WebElement]: ...
    def find_element_by_tag_name(self, name: basestring) -> WebElement: ...
    def find_elements_by_tag_name(self, name: basestring) -> List[WebElement]: ...
    def find_element_by_class_name(self, name: basestring) -> WebElement: ...
    def find_elements_by_class_name(self, name: basestring) -> List[WebElement]: ...
    def find_element_by_css_selector(self, css_selector: basestring) -> WebElement: ...
    def find_elements_by_css_selector(self, css_selector: basestring) -> List[WebElement]: ...
    def execute_script(self, script, *args): ...
    def execute_async_script(self, script, *args): ...
    @property
    def current_url(self) -> basestring: ...
    @property
    def page_source(self): ...
    def close(self): ...
    def quit(self): ...
    @property
    def current_window_handle(self): ...
    @property
    def window_handles(self): ...
    def maximize_window(self): ...
    @property
    def switch_to(self): ...
    def switch_to_active_element(self): ...
    def switch_to_window(self, window_name): ...
    def switch_to_frame(self, frame_reference): ...
    def switch_to_default_content(self): ...
    def switch_to_alert(self): ...
    def back(self): ...
    def forward(self): ...
    def refresh(self): ...
    def get_cookies(self): ...
    def get_cookie(self, name): ...
    def delete_cookie(self, name): ...
    def delete_all_cookies(self): ...
    def add_cookie(self, cookie_dict): ...
    def implicitly_wait(self, time_to_wait): ...
    def set_script_timeout(self, time_to_wait): ...
    def set_page_load_timeout(self, time_to_wait): ...
    def find_element(self, by=..., value=None): ...
    def find_elements(self, by=..., value=None): ...
    @property
    def desired_capabilities(self): ...
    def get_screenshot_as_file(self, filename): ...
    save_screenshot = ... # type: Any
    def get_screenshot_as_png(self): ...
    def get_screenshot_as_base64(self): ...
    def set_window_size(self, width, height, windowHandle=''): ...
    def get_window_size(self, windowHandle=''): ...
    def set_window_position(self, x, y, windowHandle=''): ...
    def get_window_position(self, windowHandle=''): ...
    @property
    def file_detector(self): ...
    @file_detector.setter
    def file_detector(self, detector): ...
    @property
    def orientation(self): ...
    @orientation.setter
    def orientation(self, value): ...
    @property
    def application_cache(self): ...
    @property
    def log_types(self): ...
    def get_log(self, log_type): ...



# TODO incomplete
from types import TracebackType, FrameType, ModuleType
from typing import Any, Callable, List, Optional, Tuple, NamedTuple

# Types and members
ModuleInfo = NamedTuple('ModuleInfo', [('name', str),
                                       ('suffix', str),
                                       ('mode', str),
                                       ('module_type', int),
                                       ])
def getmembers(object: object,
               predicate: Callable[[Any], bool] = ...,
               ) -> List[Tuple[str, object]]: ...
def getmoduleinfo(path: str) -> Optional[ModuleInfo]: ...
def getmodulename(path: str) -> Optional[str]: ...

def ismodule(object: object) -> bool: ...
def isclass(object: object) -> bool: ...
def ismethod(object: object) -> bool: ...
def isfunction(object: object) -> bool: ...
def isisgeneratorfunction(object: object) -> bool: ...
def isgenerator(object: object) -> bool: ...
def istraceback(object: object) -> bool: ...
def isframe(object: object) -> bool: ...
def iscode(object: object) -> bool: ...
def isbuiltin(object: object) -> bool: ...
def isroutine(object: object) -> bool: ...
def isabstract(object: object) -> bool: ...
def ismethoddescriptor(object: object) -> bool: ...
def isdatadescriptor(object: object) -> bool: ...
def isgetsetdescriptor(object: object) -> bool: ...
def ismemberdescriptor(object: object) -> bool: ...

# Retrieving source code
def getdoc(object: object) -> str: ...
def getcomments(object: object) -> str: ...
def getfile(object: object) -> str: ...
def getmodule(object: object) -> ModuleType: ...
def getsourcefile(object: object) -> str: ...
# TODO restrict to "module, class, method, function, traceback, frame,
# or code object"
def getsourcelines(object: object) -> Tuple[List[str], int]: ...
# TODO restrict to "a module, class, method, function, traceback, frame,
# or code object"
def getsource(object: object) -> str: ...
def cleandoc(doc: str) -> str: ...

# Classes and functions
# TODO make the return type more specific
def getclasstree(classes: List[type], unique: bool = ...) -> Any: ...

ArgSpec = NamedTuple('ArgSpec', [('args', List[str]),
                                 ('varargs', str),
                                 ('keywords', str),
                                 ('defaults', tuple),
                                 ])

def getargspec(func: object) -> ArgSpec: ...
# TODO make the return type more specific
def getargvalues(frame: FrameType) -> Any: ...
# TODO formatargspec
# TODO formatargvalues
def getmro(cls: type) -> Tuple[type, ...]: ...
# TODO getcallargs

# The interpreter stack
# TODO getframeinfo
# TODO getouterframes
def getinnerframes(traceback: TracebackType, context: int = ...) -> List[FrameType]:
    ...

_FrameRecord = Tuple[FrameType, str, int, str, List[str], int]

def currentframe() -> FrameType: ...
def stack(context: int = ...) -> List[_FrameRecord]: ...
def trace(context: int = ...) -> List[_FrameRecord]: ...



# Stubs for threading

from typing import Any, Optional, Callable, TypeVar, Union, List, Mapping, Sequence

def active_count() -> int: ...
def activeCount() -> int: ...

def current_thread() -> Thread: ...
def currentThread() -> Thread: ...
def enumerate() -> List[Thread]: ...

class Thread(object):
    name = ...  # type: str
    ident = 0
    daemon = False

    def __init__(self, group: Any = ..., target: Callable[..., Any] = ...,
                 name: str = ..., args: Sequence[Any] = ...,
                 kwargs: Mapping[str, Any] = ...) -> None: ...
    def start(self) -> None: ...
    def run(self) -> None: ...
    def join(self, timeout: float = ...) -> None: ...
    def is_alive(self) -> bool: ...

    # Legacy methods
    def isAlive(self) -> bool: ...
    def getName(self) -> str: ...
    def setName(self, name: str) -> None: ...
    def isDaemon(self) -> bool: ...
    def setDaemon(self, daemon: bool) -> None: ...

class Timer(Thread):
    def __init__(self, interval: float, function: Callable[..., Any],
                 args: Sequence[Any] = ...,
                 kwargs: Mapping[str, Any] = ...) -> None: ...
    def cancel(self) -> None : ...

# TODO: better type
def settrace(func: Callable[[Any, str, Any], Any]) -> None: ...
def setprofile(func: Any) -> None: ...
def stack_size(size: int = ...) -> None: ...

class ThreadError(Exception):
    pass

class local(Any): ...

class Event(object):
    def is_set(self) -> bool: ...
    def isSet(self) -> bool: ...
    def set(self) -> None: ...
    def clear(self) -> None: ...
    # TODO can it return None?
    def wait(self, timeout: float = ...) -> bool: ...

class Lock(object):
    def acquire(self, blocking: bool = ...) -> bool: ...
    def release(self) -> None: ...
    def locked(self) -> bool: ...
    def __enter__(self) -> bool: ...
    def __exit__(self, *args): ...

class RLock(object):
    def acquire(self, blocking: int = ...) -> Optional[bool]: ...
    def release(self) -> None: ...
    def __enter__(self) -> bool: ...
    def __exit__(self, *args): ...

class Semaphore(object):
    def acquire(self, blocking: bool = ...) -> Optional[bool]: ...
    def release(self) -> None: ...
    def __init__(self, value: int = ...) -> None: ...
    def __enter__(self) -> bool: ...
    def __exit__(self, *args): ...

class BoundedSemaphore(object):
    def acquire(self, blocking: bool = ...) -> Optional[bool]: ...
    def release(self) -> None: ...
    def __init__(self, value: int = ...) -> None: ...
    def __enter__(self) -> bool: ...
    def __exit__(self, *args): ...

_T = TypeVar('_T')

class Condition(object):
    def acquire(self, blocking: bool = ...) -> bool: ...
    def release(self) -> None: ...
    def notify(self, n: int = ...) -> None: ...
    def notify_all(self) -> None: ...
    def notifyAll(self) -> None: ...
    def wait(self, timeout: float = ...) -> bool: ...
    def wait_for(self, predicate: Callable[[], _T], timeout: float = ...) -> Union[_T, bool]: ...
    def __enter__(self) -> bool: ...
    def __exit__(self, *args): ...
    def __init__(self, lock: Lock = ...) -> None: ...



# Hand-written stub for lxml.etree as used by mypy.report.
# This is *far* from complete, and the stubgen-generated ones crash mypy.
# Any use of `Any` below means I couldn't figure out the type.

import typing
from typing import Any, Dict, List, Tuple, Union
from typing import SupportsBytes


# We do *not* want `typing.AnyStr` because it is a `TypeVar`, which is an
# unnecessary constraint. It seems reasonable to constrain each
# List/Dict argument to use one type consistently, though, and it is
# necessary in order to keep these brief.
AnyStr = Union[str, bytes]
ListAnyStr = Union[List[str], List[bytes]]
DictAnyStr = Union[Dict[str, str], Dict[bytes, bytes]]
Dict_Tuple2AnyStr_Any = Union[Dict[Tuple[str, str], Any], Tuple[bytes, bytes], Any]


class _Element:
    def addprevious(self, element: '_Element') -> None:
        pass

class _ElementTree:
    def write(self,
              file: Union[AnyStr, typing.IO],
              encoding: AnyStr = ...,
              method: AnyStr = ...,
              pretty_print: bool = ...,
              xml_declaration: Any = ...,
              with_tail: Any = ...,
              standalone: bool = ...,
              compression: int = ...,
              exclusive: bool = ...,
              with_comments: bool = ...,
              inclusive_ns_prefixes: ListAnyStr = ...) -> None:
        pass

class _XSLTResultTree(SupportsBytes):
    pass

class _XSLTQuotedStringParam:
    pass

class XMLParser:
    pass

class XMLSchema:
    def __init__(self,
                 etree: Union[_Element, _ElementTree] = ...,
                 file: Union[AnyStr, typing.IO] = ...) -> None:
        pass

    def assertValid(self,
                    etree: Union[_Element, _ElementTree]) -> None:
        pass

class XSLTAccessControl:
    pass

class XSLT:
    def __init__(self,
                 xslt_input: Union[_Element, _ElementTree],
                 extensions: Dict_Tuple2AnyStr_Any = ...,
                 regexp: bool = ...,
                 access_control: XSLTAccessControl = ...) -> None:
        pass

    def __call__(self,
                 _input: Union[_Element, _ElementTree],
                 profile_run: bool = ...,
                 **kwargs: Union[AnyStr, _XSLTQuotedStringParam]) -> _XSLTResultTree:
        pass

    @staticmethod
    def strparam(s: AnyStr) -> _XSLTQuotedStringParam:
        pass

def Element(_tag: AnyStr,
            attrib: DictAnyStr = ...,
            nsmap: DictAnyStr = ...,
            **extra: AnyStr) -> _Element:
    pass

def SubElement(_parent: _Element, _tag: AnyStr,
               attrib: DictAnyStr = ...,
               nsmap: DictAnyStr = ...,
               **extra: AnyStr) -> _Element:
    pass

def ElementTree(element: _Element = ...,
                file: Union[AnyStr, typing.IO] = ...,
                parser: XMLParser = ...) -> _ElementTree:
    pass

def ProcessingInstruction(target: AnyStr, text: AnyStr = ...) -> _Element:
    pass

def parse(source: Union[AnyStr, typing.IO],
          parser: XMLParser = ...,
          base_url: AnyStr = ...) -> _ElementTree:
    pass



# Stubs for sqlalchemy.dialects.mysql.base (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from ... import sql
from ... import engine
from ... import util
from ... import types

sqltypes = sql.sqltypes
## compiler = sql.compiler
## reflection = engine.reflection
## default = engine.default
## topological = util.topological
DATE = types.DATE
BOOLEAN = types.BOOLEAN
BLOB = types.BLOB
BINARY = types.BINARY
VARBINARY = types.VARBINARY

RESERVED_WORDS = ... # type: Any
AUTOCOMMIT_RE = ... # type: Any
SET_RE = ... # type: Any

class _NumericType:
    unsigned = ... # type: Any
    zerofill = ... # type: Any
    def __init__(self, unsigned=..., zerofill=..., **kw) -> None: ...

class _FloatType(_NumericType,
                 ## sqltypes.Float
                 ):
    scale = ... # type: Any
    def __init__(self, precision=..., scale=..., asdecimal=..., **kw) -> None: ...

class _IntegerType(_NumericType,
                   ## sqltypes.Integer
                   ):
    display_width = ... # type: Any
    def __init__(self, display_width=..., **kw) -> None: ...

class _StringType(object,
                  ## sqltypes.String
                  ):
    charset = ... # type: Any
    ascii = ... # type: Any
    unicode = ... # type: Any
    binary = ... # type: Any
    national = ... # type: Any
    def __init__(self, charset=..., collation=..., ascii=..., binary=..., unicode=..., national=..., **kw) -> None: ...

class _MatchType(object,
                 ## sqltypes.Float,
                 ## sqltypes.MatchType
                 ):
    def __init__(self, **kw) -> None: ...

class NUMERIC(_NumericType,
              ## sqltypes.NUMERIC
              ):
    __visit_name__ = ... # type: Any
    def __init__(self, precision=..., scale=..., asdecimal=..., **kw) -> None: ...

class DECIMAL(_NumericType,
              ## sqltypes.DECIMAL
              ):
    __visit_name__ = ... # type: Any
    def __init__(self, precision=..., scale=..., asdecimal=..., **kw) -> None: ...

class DOUBLE(_FloatType):
    __visit_name__ = ... # type: Any
    def __init__(self, precision=..., scale=..., asdecimal=..., **kw) -> None: ...

class REAL(_FloatType,
           ## sqltypes.REAL
           ):
    __visit_name__ = ... # type: Any
    def __init__(self, precision=..., scale=..., asdecimal=..., **kw) -> None: ...

class FLOAT(_FloatType,
            ## sqltypes.FLOAT
            ):
    __visit_name__ = ... # type: Any
    def __init__(self, precision=..., scale=..., asdecimal=..., **kw) -> None: ...
    def bind_processor(self, dialect): ...

class INTEGER(_IntegerType,
              ## sqltypes.INTEGER
              ):
    __visit_name__ = ... # type: Any
    def __init__(self, display_width=..., **kw) -> None: ...

class BIGINT(_IntegerType,
             ## sqltypes.BIGINT
             ):
    __visit_name__ = ... # type: Any
    def __init__(self, display_width=..., **kw) -> None: ...

class MEDIUMINT(_IntegerType):
    __visit_name__ = ... # type: Any
    def __init__(self, display_width=..., **kw) -> None: ...

class TINYINT(_IntegerType):
    __visit_name__ = ... # type: Any
    def __init__(self, display_width=..., **kw) -> None: ...

class SMALLINT(_IntegerType,
               ## sqltypes.SMALLINT
               ):
    __visit_name__ = ... # type: Any
    def __init__(self, display_width=..., **kw) -> None: ...

class BIT(object,
          ## sqltypes.TypeEngine
          ):
    __visit_name__ = ... # type: Any
    length = ... # type: Any
    def __init__(self, length=...) -> None: ...
    def result_processor(self, dialect, coltype): ...

class TIME(object,
           ## sqltypes.TIME
           ):
    __visit_name__ = ... # type: Any
    fsp = ... # type: Any
    def __init__(self, timezone=..., fsp=...) -> None: ...
    def result_processor(self, dialect, coltype): ...

class TIMESTAMP(object,
                ## sqltypes.TIMESTAMP
                ):
    __visit_name__ = ... # type: Any
    fsp = ... # type: Any
    def __init__(self, timezone=..., fsp=...) -> None: ...

class DATETIME(object,
               ## sqltypes.DATETIME
               ):
    __visit_name__ = ... # type: Any
    fsp = ... # type: Any
    def __init__(self, timezone=..., fsp=...) -> None: ...

class YEAR(object,
           ## sqltypes.TypeEngine
           ):
    __visit_name__ = ... # type: Any
    display_width = ... # type: Any
    def __init__(self, display_width=...) -> None: ...

class TEXT(_StringType,
           ## sqltypes.TEXT
           ):
    __visit_name__ = ... # type: Any
    def __init__(self, length=..., **kw) -> None: ...

class TINYTEXT(_StringType):
    __visit_name__ = ... # type: Any
    def __init__(self, **kwargs) -> None: ...

class MEDIUMTEXT(_StringType):
    __visit_name__ = ... # type: Any
    def __init__(self, **kwargs) -> None: ...

class LONGTEXT(_StringType):
    __visit_name__ = ... # type: Any
    def __init__(self, **kwargs) -> None: ...

class VARCHAR(_StringType,
              ## sqltypes.VARCHAR
              ):
    __visit_name__ = ... # type: Any
    def __init__(self, length=..., **kwargs) -> None: ...

class CHAR(_StringType,
           ## sqltypes.CHAR
           ):
    __visit_name__ = ... # type: Any
    def __init__(self, length=..., **kwargs) -> None: ...

class NVARCHAR(_StringType,
               ## sqltypes.NVARCHAR
               ):
    __visit_name__ = ... # type: Any
    def __init__(self, length=..., **kwargs) -> None: ...

class NCHAR(_StringType,
            ## sqltypes.NCHAR
            ):
    __visit_name__ = ... # type: Any
    def __init__(self, length=..., **kwargs) -> None: ...

class TINYBLOB(object,
               ## sqltypes._Binary
               ):
    __visit_name__ = ... # type: Any

class MEDIUMBLOB(object,
                 ## sqltypes._Binary
                 ):
    __visit_name__ = ... # type: Any

class LONGBLOB(object,
               ## sqltypes._Binary
               ):
    __visit_name__ = ... # type: Any

class _EnumeratedValues(_StringType): ...

class ENUM(## sqltypes.Enum,
           _EnumeratedValues
           ):
    __visit_name__ = ... # type: Any
    strict = ... # type: Any
    def __init__(self, *enums, **kw) -> None: ...
    def bind_processor(self, dialect): ...
    def adapt(self, cls, **kw): ...

class SET(_EnumeratedValues):
    __visit_name__ = ... # type: Any
    retrieve_as_bitwise = ... # type: Any
    values = ... # type: Any
    def __init__(self, *values, **kw) -> None: ...
    def column_expression(self, colexpr): ...
    def result_processor(self, dialect, coltype): ...
    def bind_processor(self, dialect): ...
    def adapt(self, impltype, **kw): ...

MSTime = ... # type: Any
MSSet = ... # type: Any
MSEnum = ... # type: Any
MSLongBlob = ... # type: Any
MSMediumBlob = ... # type: Any
MSTinyBlob = ... # type: Any
MSBlob = ... # type: Any
MSBinary = ... # type: Any
MSVarBinary = ... # type: Any
MSNChar = ... # type: Any
MSNVarChar = ... # type: Any
MSChar = ... # type: Any
MSString = ... # type: Any
MSLongText = ... # type: Any
MSMediumText = ... # type: Any
MSTinyText = ... # type: Any
MSText = ... # type: Any
MSYear = ... # type: Any
MSTimeStamp = ... # type: Any
MSBit = ... # type: Any
MSSmallInteger = ... # type: Any
MSTinyInteger = ... # type: Any
MSMediumInteger = ... # type: Any
MSBigInteger = ... # type: Any
MSNumeric = ... # type: Any
MSDecimal = ... # type: Any
MSDouble = ... # type: Any
MSReal = ... # type: Any
MSFloat = ... # type: Any
MSInteger = ... # type: Any
colspecs = ... # type: Any
ischema_names = ... # type: Any

class MySQLExecutionContext(object,
                            ## default.DefaultExecutionContext
                            ):
    def should_autocommit_text(self, statement): ...

class MySQLCompiler(object,
                    ## compiler.SQLCompiler
                    ):
    render_table_with_column_in_update_from = ... # type: Any
    extract_map = ... # type: Any
    def visit_random_func(self, fn, **kw): ...
    def visit_utc_timestamp_func(self, fn, **kw): ...
    def visit_sysdate_func(self, fn, **kw): ...
    def visit_concat_op_binary(self, binary, operator, **kw): ...
    def visit_match_op_binary(self, binary, operator, **kw): ...
    def get_from_hint_text(self, table, text): ...
    def visit_typeclause(self, typeclause, type_=...): ...
    def visit_cast(self, cast, **kwargs): ...
    def render_literal_value(self, value, type_): ...
    def visit_true(self, element, **kw): ...
    def visit_false(self, element, **kw): ...
    def get_select_precolumns(self, select, **kw): ...
    def visit_join(self, join, asfrom=..., **kwargs): ...
    def for_update_clause(self, select, **kw): ...
    def limit_clause(self, select, **kw): ...
    def update_limit_clause(self, update_stmt): ...
    def update_tables_clause(self, update_stmt, from_table, extra_froms, **kw): ...
    def update_from_clause(self, update_stmt, from_table, extra_froms, from_hints, **kw): ...

class MySQLDDLCompiler(object,
                       ## compiler.DDLCompiler
                       ):
    def create_table_constraints(self, table, **kw): ...
    def get_column_specification(self, column, **kw): ...
    def post_create_table(self, table): ...
    def visit_create_index(self, create): ...
    def visit_primary_key_constraint(self, constraint): ...
    def visit_drop_index(self, drop): ...
    def visit_drop_constraint(self, drop): ...
    def define_constraint_match(self, constraint): ...

class MySQLTypeCompiler(object,
                        ## compiler.GenericTypeCompiler
                        ):
    def visit_NUMERIC(self, type_, **kw): ...
    def visit_DECIMAL(self, type_, **kw): ...
    def visit_DOUBLE(self, type_, **kw): ...
    def visit_REAL(self, type_, **kw): ...
    def visit_FLOAT(self, type_, **kw): ...
    def visit_INTEGER(self, type_, **kw): ...
    def visit_BIGINT(self, type_, **kw): ...
    def visit_MEDIUMINT(self, type_, **kw): ...
    def visit_TINYINT(self, type_, **kw): ...
    def visit_SMALLINT(self, type_, **kw): ...
    def visit_BIT(self, type_, **kw): ...
    def visit_DATETIME(self, type_, **kw): ...
    def visit_DATE(self, type_, **kw): ...
    def visit_TIME(self, type_, **kw): ...
    def visit_TIMESTAMP(self, type_, **kw): ...
    def visit_YEAR(self, type_, **kw): ...
    def visit_TEXT(self, type_, **kw): ...
    def visit_TINYTEXT(self, type_, **kw): ...
    def visit_MEDIUMTEXT(self, type_, **kw): ...
    def visit_LONGTEXT(self, type_, **kw): ...
    def visit_VARCHAR(self, type_, **kw): ...
    def visit_CHAR(self, type_, **kw): ...
    def visit_NVARCHAR(self, type_, **kw): ...
    def visit_NCHAR(self, type_, **kw): ...
    def visit_VARBINARY(self, type_, **kw): ...
    def visit_large_binary(self, type_, **kw): ...
    def visit_enum(self, type_, **kw): ...
    def visit_BLOB(self, type_, **kw): ...
    def visit_TINYBLOB(self, type_, **kw): ...
    def visit_MEDIUMBLOB(self, type_, **kw): ...
    def visit_LONGBLOB(self, type_, **kw): ...
    def visit_ENUM(self, type_, **kw): ...
    def visit_SET(self, type_, **kw): ...
    def visit_BOOLEAN(self, type, **kw): ...

class MySQLIdentifierPreparer(object,
                              ## compiler.IdentifierPreparer
                              ):
    reserved_words = ... # type: Any
    def __init__(self, dialect, server_ansiquotes=..., **kw) -> None: ...

class MySQLDialect(object,
                   ## default.DefaultDialect
                   ):
    name = ... # type: Any
    supports_alter = ... # type: Any
    supports_native_boolean = ... # type: Any
    max_identifier_length = ... # type: Any
    max_index_name_length = ... # type: Any
    supports_native_enum = ... # type: Any
    supports_sane_rowcount = ... # type: Any
    supports_sane_multi_rowcount = ... # type: Any
    supports_multivalues_insert = ... # type: Any
    default_paramstyle = ... # type: Any
    colspecs = ... # type: Any
    statement_compiler = ... # type: Any
    ddl_compiler = ... # type: Any
    type_compiler = ... # type: Any
    ischema_names = ... # type: Any
    preparer = ... # type: Any
    construct_arguments = ... # type: Any
    isolation_level = ... # type: Any
    def __init__(self, isolation_level=..., **kwargs) -> None: ...
    def on_connect(self): ...
    def set_isolation_level(self, connection, level): ...
    def get_isolation_level(self, connection): ...
    def do_commit(self, dbapi_connection): ...
    def do_rollback(self, dbapi_connection): ...
    def do_begin_twophase(self, connection, xid): ...
    def do_prepare_twophase(self, connection, xid): ...
    def do_rollback_twophase(self, connection, xid, is_prepared=..., recover=...): ...
    def do_commit_twophase(self, connection, xid, is_prepared=..., recover=...): ...
    def do_recover_twophase(self, connection): ...
    def is_disconnect(self, e, connection, cursor): ...
    def has_table(self, connection, table_name, schema=...): ...
    identifier_preparer = ... # type: Any
    def initialize(self, connection): ...
    def get_schema_names(self, connection, **kw): ...
    def get_table_names(self, connection, schema=..., **kw): ...
    def get_view_names(self, connection, schema=..., **kw): ...
    def get_table_options(self, connection, table_name, schema=..., **kw): ...
    def get_columns(self, connection, table_name, schema=..., **kw): ...
    def get_pk_constraint(self, connection, table_name, schema=..., **kw): ...
    def get_foreign_keys(self, connection, table_name, schema=..., **kw): ...
    def get_indexes(self, connection, table_name, schema=..., **kw): ...
    def get_unique_constraints(self, connection, table_name, schema=..., **kw): ...
    def get_view_definition(self, connection, view_name, schema=..., **kw): ...

class ReflectedState:
    columns = ... # type: Any
    table_options = ... # type: Any
    table_name = ... # type: Any
    keys = ... # type: Any
    constraints = ... # type: Any
    def __init__(self) -> None: ...

class MySQLTableDefinitionParser:
    dialect = ... # type: Any
    preparer = ... # type: Any
    def __init__(self, dialect, preparer) -> None: ...
    def parse(self, show_create, charset): ...

class _DecodingRowProxy:
    rowproxy = ... # type: Any
    charset = ... # type: Any
    def __init__(self, rowproxy, charset) -> None: ...
    def __getitem__(self, index): ...
    def __getattr__(self, attr): ...



# Stubs for email.policy (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
import email._policybase

Policy = email._policybase.Policy
Compat32 = email._policybase.Compat32

class EmailPolicy(Policy):
    refold_source = ...  # type: Any
    header_factory = ...  # type: Any
    content_manager = ...  # type: Any
    def __init__(self, **kw) -> None: ...
    def header_max_count(self, name): ...
    def header_source_parse(self, sourcelines): ...
    def header_store_parse(self, name, value): ...
    def header_fetch_parse(self, name, value): ...
    def fold(self, name, value): ...
    def fold_binary(self, name, value): ...

default = ...  # type: Any
strict = ...  # type: Any
SMTP = ...  # type: Any
HTTP = ...  # type: Any



# Stubs for numbers (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class Number:
    __hash__ = ... # type: Any

class Complex(Number):
    def __complex__(self): ...
    def __bool__(self): ...
    @property
    def real(self): ...
    @property
    def imag(self): ...
    def __add__(self, other): ...
    def __radd__(self, other): ...
    def __neg__(self): ...
    def __pos__(self): ...
    def __sub__(self, other): ...
    def __rsub__(self, other): ...
    def __mul__(self, other): ...
    def __rmul__(self, other): ...
    def __truediv__(self, other): ...
    def __rtruediv__(self, other): ...
    def __pow__(self, exponent): ...
    def __rpow__(self, base): ...
    def __abs__(self): ...
    def conjugate(self): ...
    def __eq__(self, other): ...

class Real(Complex):
    def __float__(self): ...
    def __trunc__(self): ...
    def __floor__(self): ...
    def __ceil__(self): ...
    def __round__(self, ndigits=None): ...
    def __divmod__(self, other): ...
    def __rdivmod__(self, other): ...
    def __floordiv__(self, other): ...
    def __rfloordiv__(self, other): ...
    def __mod__(self, other): ...
    def __rmod__(self, other): ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __complex__(self): ...
    @property
    def real(self): ...
    @property
    def imag(self): ...
    def conjugate(self): ...

class Rational(Real):
    @property
    def numerator(self): ...
    @property
    def denominator(self): ...
    def __float__(self): ...

class Integral(Rational):
    def __int__(self): ...
    def __index__(self): ...
    def __pow__(self, exponent, modulus=None): ...
    def __lshift__(self, other): ...
    def __rlshift__(self, other): ...
    def __rshift__(self, other): ...
    def __rrshift__(self, other): ...
    def __and__(self, other): ...
    def __rand__(self, other): ...
    def __xor__(self, other): ...
    def __rxor__(self, other): ...
    def __or__(self, other): ...
    def __ror__(self, other): ...
    def __invert__(self): ...
    def __float__(self): ...
    @property
    def numerator(self): ...
    @property
    def denominator(self): ...



# Stubs for traceback

from typing import List
from types import TracebackType
import typing

# TODO signatures
def format_exception_only(etype, value): ...
def format_exception(type: type, value: List[str], tb: TracebackType, limit: int, chain: bool) -> str: ...
def format_tb(traceback): ...
def print_exc(limit=..., file=..., chain=...): ...
def format_exc(limit: int = ..., chain: bool = ...) -> str: ...
def extract_stack(f=..., limit=...): ...
def extract_tb(traceback, limit=...): ...
def format_list(list): ...
def format_stack(f=..., limit=...) -> List[str]: ...

# TODO add more



# Stubs for xml.etree.ElementTree (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, AnyStr, Union, IO, Callable, Dict, List, Tuple, Sequence, Iterator, TypeVar, Optional, KeysView, ItemsView, Generator
import io

VERSION = ... # type: str

_Ss = TypeVar('_Ss', str, bytes)
_T = TypeVar('_T')
_str_or_bytes = Union[str, bytes]

class _SimpleElementPath:
    def find(self, element: 'Element', tag: _str_or_bytes, namespaces: Any=...) -> Optional['Element']: ...
    def findtext(self, element: 'Element', tag: _str_or_bytes, default: _T=..., namespaces: Any=...) -> Union[str, bytes, _T]: ...
    def iterfind(self, element: 'Element', tag: _str_or_bytes, namespaces: Any=...) -> Generator['Element', None, None]: ...
    def findall(self, element: 'Element', tag: _str_or_bytes, namespaces: Any=...) -> List['Element']: ...

class ParseError(SyntaxError): ...

def iselement(element: 'Element') -> bool: ...

class Element:
    tag = ... # type: _str_or_bytes
    attrib = ... # type: Dict[_str_or_bytes, _str_or_bytes]
    text = ... # type: Optional[_str_or_bytes]
    tail = ... # type: Optional[_str_or_bytes]
    def __init__(self, tag: Union[AnyStr, Callable[..., 'Element']], attrib: Dict[AnyStr, AnyStr]=..., **extra: Dict[str, AnyStr]) -> None: ...
    def append(self, subelement: 'Element') -> None: ...
    def clear(self) -> None: ...
    def copy(self) -> 'Element': ...
    def extend(self, elements: Sequence['Element']) -> None: ...
    def find(self, path: str, namespaces: Dict[str, str]=...) -> Optional['Element']: ...
    def findall(self, path: str, namespaces: Dict[str, str]=...) -> List['Element']: ...
    def findtext(self, path: str, default: _T=..., namespaces: Dict[str, str]=...) -> Union[_T, str]: ...
    def get(self, key: AnyStr, default: _T=...) -> Union[AnyStr, _T]: ...
    def getchildren(self) -> List['Element']: ...
    def getiterator(self, tag: Union[str, AnyStr]=...) -> List['Element']: ...
    def insert(self, index: int, subelement: 'Element') -> None: ...
    def items(self) -> ItemsView[AnyStr, AnyStr]: ...
    def iter(self, tag: Union[str, AnyStr]=...) -> Generator['Element', None, None]: ...
    def iterfind(self, path: str, namespaces: Dict[str, str]=...) -> List['Element']: ...
    def itertext(self) -> Generator[str, None, None]: ...
    def keys(self) -> KeysView[AnyStr]: ...
    def makeelement(self, tag: _Ss, attrib: Dict[_Ss, _Ss]) -> 'Element': ...
    def remove(self, subelement: 'Element') -> None: ...
    def set(self, key: AnyStr, value: AnyStr) -> None: ...
    def __bool__(self) -> bool: ...
    def __delitem__(self, index: int) -> None: ...
    def __getitem__(self, index) -> 'Element': ...
    def __len__(self) -> int: ...
    def __setitem__(self, index: int, element: 'Element') -> None: ...

def SubElement(parent: Element, tag: AnyStr, attrib: Dict[AnyStr, AnyStr]=..., **extra: Dict[str, AnyStr]) -> Element: ...
def Comment(text: _str_or_bytes=...) -> Element: ...
def ProcessingInstruction(target: str, text: str=...) -> Element: ...

PI = ... # type: Callable[..., Element]

class QName:
    text = ... # type: str
    def __init__(self, text_or_uri: str, tag: str=...) -> None: ...


_file_or_filename = Union[str, bytes, int, IO[Any]]

class ElementTree:
    def __init__(self, element: Element=..., file: _file_or_filename=...) -> None: ...
    def getroot(self) -> Element: ...
    def parse(self, source: _file_or_filename, parser: 'XMLParser'=...) -> Element: ...
    def iter(self, tag: Union[str, AnyStr]=...) -> Generator[Element, None, None]: ...
    def getiterator(self, tag: Union[str, AnyStr]=...) -> List[Element]: ...
    def find(self, path: str, namespaces: Dict[str, str]=...) -> Optional[Element]: ...
    def findtext(self, path: str, default: _T=..., namespaces: Dict[str, str]=...) -> Union[_T, str]: ...
    def findall(self, path: str, namespaces: Dict[str, str]=...) -> List[Element]: ...
    def iterfind(self, path: str, namespaces: Dict[str, str]=...) -> List[Element]: ...
    def write(self, file_or_filename: _file_or_filename, encoding: str=..., xml_declaration: Optional[bool]=..., default_namespace: str=..., method: str=...) -> None: ...
    def write_c14n(self, file: _file_or_filename) -> None: ...

def register_namespace(prefix: str, uri: str) -> None: ...
def tostring(element: Element, encoding: str=..., method: str=...) -> str: ...

def tostringlist(element: Element, encoding: str=..., method: str=...) -> List[str]: ...
def dump(elem: Element) -> None: ...
def parse(source: _file_or_filename, parser: 'XMLParser'=...) -> ElementTree: ...
def iterparse(source: _file_or_filename, events: Sequence[str]=..., parser: 'XMLParser'=...) -> Iterator[Tuple[str, Element]]: ...


class _IterParseIterator:
    root = ... # type: Any
    def __init__(self, source: _file_or_filename, events: Sequence[str], parser: 'XMLParser', close_source: bool=...) -> None: ...
    def __next__(self) -> Tuple[str, Element]: ...
    def __iter__(self) -> _IterParseIterator: ...

def XML(text: AnyStr, parser: 'XMLParser'=...) -> Element: ...
def XMLID(text: AnyStr, parser: 'XMLParser'=...) -> Tuple[Element, Dict[str, Element]]: ...

# TODO-improve this type
fromstring = ... # type: Callable[..., Element]

def fromstringlist(sequence: Sequence[AnyStr], parser: 'XMLParser'=...) -> Element: ...

class TreeBuilder:
    def __init__(self, element_factory: Callable[[AnyStr, Dict[AnyStr, AnyStr]], Element]=...) -> None: ...
    def close(self) -> Element: ...
    def data(self, data: AnyStr) -> None: ...
    def start(self, tag: AnyStr, attrs: Dict[AnyStr, AnyStr]) -> Element: ...
    def end(self, tag: AnyStr) -> Element: ...

class XMLParser:
    parser = ... # type: Any
    target = ... # type: TreeBuilder
    # TODO-what is entity used for???
    entity = ... # type: Any
    version = ... # type: str
    def __init__(self, html: int=..., target: TreeBuilder=..., encoding: str=...) -> None: ...
    def doctype(self, name: str, pubid: str, system: str) -> None: ...
    def close(self) -> Any: ...  # TODO-most of the time, this will be Element, but it can be anything target.close() returns
    def feed(self, data: AnyStr)-> None: ...



# Stubs for scribe.ttypes (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

fastbinary = ... # type: Any

class ResultCode:
    OK = ... # type: Any
    TRY_LATER = ... # type: Any

class LogEntry:
    thrift_spec = ... # type: Any
    category = ... # type: Any
    message = ... # type: Any
    def __init__(self, category=..., message=...) -> None: ...
    def read(self, iprot): ...
    def write(self, oprot): ...
    def validate(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



# Stubs for boto (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
import logging

Version = ... # type: Any
UserAgent = ... # type: Any
config = ... # type: Any
BUCKET_NAME_RE = ... # type: Any
TOO_LONG_DNS_NAME_COMP = ... # type: Any
GENERATION_RE = ... # type: Any
VERSION_RE = ... # type: Any
ENDPOINTS_PATH = ... # type: Any

def init_logging(): ...

class NullHandler(logging.Handler):
    def emit(self, record): ...

log = ... # type: Any
perflog = ... # type: Any

def set_file_logger(name, filepath, level=..., format_string=...): ...
def set_stream_logger(name, level=..., format_string=...): ...
def connect_sqs(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_s3(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_gs(gs_access_key_id=..., gs_secret_access_key=..., **kwargs): ...
def connect_ec2(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_elb(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_autoscale(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cloudwatch(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_sdb(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_fps(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_mturk(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cloudfront(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_vpc(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_rds(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_rds2(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_emr(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_sns(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_iam(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_route53(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cloudformation(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_euca(host=..., aws_access_key_id=..., aws_secret_access_key=..., port=..., path=..., is_secure=..., **kwargs): ...
def connect_glacier(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_ec2_endpoint(url, aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_walrus(host=..., aws_access_key_id=..., aws_secret_access_key=..., port=..., path=..., is_secure=..., **kwargs): ...
def connect_ses(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_sts(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_ia(ia_access_key_id=..., ia_secret_access_key=..., is_secure=..., **kwargs): ...
def connect_dynamodb(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_swf(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cloudsearch(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cloudsearch2(aws_access_key_id=..., aws_secret_access_key=..., sign_request=..., **kwargs): ...
def connect_cloudsearchdomain(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_beanstalk(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_elastictranscoder(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_opsworks(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_redshift(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_support(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cloudtrail(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_directconnect(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_kinesis(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_logs(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_route53domains(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cognito_identity(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cognito_sync(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_kms(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_awslambda(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_codedeploy(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_configservice(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cloudhsm(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_ec2containerservice(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_machinelearning(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def storage_uri(uri_str, default_scheme=..., debug=..., validate=..., bucket_storage_uri_class=..., suppress_consec_slashes=..., is_latest=...): ...
def storage_uri_for_key(key): ...



from typing import Optional, List

class struct_group(object):
    gr_name = ...  # type: Optional[str]
    gr_passwd = ...  # type: Optional[str]
    gr_gid = ...  # type: int
    gr_mem = ...  # type: List[str]

def getgrall() -> List[struct_group]: ...
def getgrgid(id: int) -> struct_group: ...
def getgrnam(name: str) -> struct_group: ...



# Better textwrap stubs hand-written by o11c.
# https://docs.python.org/3/library/textwrap.html
from typing import (
        Callable,
        List,
)

class TextWrapper:
    def __init__(self,
            width: int = ...,
            *,
            initial_indent: str = ...,
            subsequent_indent: str = ...,
            expand_tabs: bool = ...,
            tabsize: int = ...,
            replace_whitespace: bool = ...,
            fix_sentence_endings: bool = ...,
            break_long_words: bool = ...,
            break_on_hyphens: bool = ...,
            drop_whitespace: bool = ...,
            max_lines: int = ...,
            placeholder: str = ...
    ) -> None:
        self.width = width
        self.initial_indent = initial_indent
        self.subsequent_indent = subsequent_indent
        self.expand_tabs = expand_tabs
        self.tabsize = tabsize
        self.replace_whitespace = replace_whitespace
        self.fix_sentence_endings = fix_sentence_endings
        self.break_long_words = break_long_words
        self.break_on_hyphens = break_on_hyphens
        self.drop_whitespace = drop_whitespace
        self.max_lines = max_lines
        self.placeholder = placeholder

    # Private methods *are* part of the documented API for subclasses.
    def _munge_whitespace(self, text: str) -> str:
        ...

    def _split(self, text: str) -> List[str]:
        ...

    def _fix_sentence_endings(self, chunks: List[str]) -> None:
        ...

    def _handle_long_word(self, reversed_chunks: List[str], cur_line: List[str], cur_len: int, width: int) -> None:
        ...

    def _wrap_chunks(self, chunks: List[str]) -> List[str]:
        ...

    def _split_chunks(self, text: str) -> List[str]:
        ...

    def wrap(self, text: str) -> List[str]:
        ...

    def fill(self, text: str) -> str:
        ...


def wrap(
        text: str = ...,
        width: int = ...,
        *,
        initial_indent: str = ...,
        subsequent_indent: str = ...,
        expand_tabs: bool = ...,
        tabsize: int = ...,
        replace_whitespace: bool = ...,
        fix_sentence_endings: bool = ...,
        break_long_words: bool = ...,
        break_on_hyphens: bool = ...,
        drop_whitespace: bool = ...,
        max_lines: int = ...,
        placeholder: str = ...
) -> List[str]:
    ...

def fill(
        width: int = ...,
        *,
        initial_indent: str = ...,
        subsequent_indent: str = ...,
        expand_tabs: bool = ...,
        tabsize: int = ...,
        replace_whitespace: bool = ...,
        fix_sentence_endings: bool = ...,
        break_long_words: bool = ...,
        break_on_hyphens: bool = ...,
        drop_whitespace: bool = ...,
        max_lines: int = ...,
        placeholder: str = ...
):
    ...

def shorten(
        width: int,
        *,
        initial_indent: str = ...,
        subsequent_indent: str = ...,
        expand_tabs: bool = ...,
        tabsize: int = ...,
        replace_whitespace: bool = ...,
        fix_sentence_endings: bool = ...,
        break_long_words: bool = ...,
        break_on_hyphens: bool = ...,
        drop_whitespace: bool = ...,
        # Omit `max_lines: int = None`, it is forced to 1 here.
        placeholder: str = ...
):
    ...

def dedent(text: str) -> str:
    ...

def indent(text: str, prefix: str, predicate: Callable[[str], bool] = ...) -> str:
    ...



# TODO these are incomplete

from typing import Any, List, overload, Callable

class TarError(Exception): ...

class TarInfo:
    name = ...  # type: str
    size = 0
    uid = 0
    gid = 0

class TarFile:
    def getmember(self, name: str) -> TarInfo: ...
    def getmembers(self) -> List[TarInfo]: ...
    def getnames(self) -> List[str]: ...
    def extractall(self, path: str = ...,
                   members: List[TarInfo] = ...) -> None: ...

    @overload
    def extract(self, member: str, path: str = ...,
                set_attrs: bool = ...) -> None: ...
    @overload
    def extract(self, member: TarInfo, path: str = ...,
                set_attrs: bool = ...) -> None: ...

    def add(self, name: str, arcname: str = ..., recursive: bool = ...,
            exclude: Callable[[str], bool] = ..., *,
            filter: 'Callable[[TarFile], TarFile]' = ...) -> None: ...
    def close(self) -> None: ...

def open(name: str = ..., mode: str = ..., fileobj: Any = ..., bufsize: int = ...,
         **kwargs) -> TarFile: ...



from typing import Any, Optional, Iterable, Tuple, List, Union

DEFAULT_BUFFER_SIZE = ...  # type: int


class BlockingIOError(IOError):
    characters_written = ...  # type: int

class UnsupportedOperation(ValueError, IOError): ...


class _IOBase(object):
    closed = ...  # type: bool
    def __enter__(self) -> "_IOBase": ...
    def __exit__(self, type, value, traceback) -> bool: ...
    def __iter__(self) -> "_IOBase": ...
    def _checkClosed(self) -> None: ...
    def _checkReadable(self) -> None: ...
    def _checkSeekable(self) -> None: ...
    def _checkWritable(self) -> None: ...
    def close(self) -> None: ...
    def fileno(self) -> int: ...
    def flush(self) -> None: ...
    def isatty(self) -> bool: ...
    def next(self) -> str: ...
    def readable(self) -> bool: ...
    def readline(self, limit: int = ...) -> str: ...
    def readlines(self, hint: int = ...) -> List[str]: ...
    def seek(self, offset: int, whence: int = ...) -> None: ...
    def seekable(self) -> bool: ...
    def tell(self) -> int: ...
    def truncate(self, size: int = ...) -> int: ...
    def writable(self) -> bool: ...
    def writelines(self, lines: Iterable[str]) -> None: ...

class _BufferedIOBase(_IOBase):
    def read1(self, n: int) -> str: ...
    def read(self, n: int = ...) -> str: ...
    def readinto(self, buffer: bytearray) -> int: ...
    def write(self, s: str) -> int: ...
    def detach(self) -> "_BufferedIOBase": ...

class BufferedRWPair(_BufferedIOBase):
    def peek(self, n: int = ...) -> str: ...

class BufferedRandom(_BufferedIOBase):
    name = ...  # type: str
    raw = ...  # type: _IOBase
    mode = ...  # type: str
    def peek(self, n: int = ...) -> str: ...

class BufferedReader(_BufferedIOBase):
    name = ...  # type: str
    raw = ...  # type: _IOBase
    mode = ...  # type: str
    def peek(self, n: int = ...) -> str: ...

class BufferedWriter(_BufferedIOBase):
    name = ...  # type: str
    raw = ...  # type: _IOBase
    mode = ...  # type: str

class BytesIO(_BufferedIOBase):
    def __setstate__(self, tuple) -> None: ...
    def __getstate__(self) -> tuple: ...
    def getvalue(self) -> str: ...

class _RawIOBase(_IOBase):
    def readall(self) -> str: ...
    def read(self, n: int = ...) -> str: ...

class FileIO(_RawIOBase):
    mode = ...  # type: str
    closefd = ...  # type: bool
    def readinto(self, buffer: bytearray)-> int: ...
    def write(self, pbuf: str) -> int: ...

class IncrementalNewlineDecoder(object):
    newlines = ...  # type: Union[str, unicode]
    def decode(self, input, final) -> Any: ...
    def getstate(self) -> Tuple[Any, int]: ...
    def setstate(self, state: Tuple[Any, int]) -> None: ...
    def reset(self) -> None: ...

class _TextIOBase(_IOBase):
    errors = ...  # type: Optional[str]
    newlines = ...  # type: Union[str, unicode]
    encoding = ...  # type: Optional[str]
    def read(self, n: int = ...) -> str: ...
    def write(self) -> None:
        raise UnsupportedOperation
    def detach(self) -> None:
        raise UnsupportedOperation

class StringIO(_TextIOBase):
    line_buffering = ...  # type: bool
    def getvalue(self) -> str: ...
    def __setstate__(self, state: tuple) -> None: ...
    def __getstate__(self) -> tuple: ...

class TextIOWrapper(_TextIOBase):
    name = ...  # type: str
    line_buffering = ...  # type: bool
    buffer = ...  # type: str
    _CHUNK_SIZE = ...  # type: int

def open(file: Union[int, str], mode: str = ...) -> _IOBase: ...
